# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Operation(api_types.Base):
    """An Operation resource represents an operation or action.

    This is for defining actions that may change the state of the resource they
    are related to. For example, the API already provides ways to register,
    start, and stop your application (POST an Assembly to register+start, and
    DELETE an Assembly to stop) but Operations provide a way to extend the
    system to add your own actions such as "pause" and "resume", or "scale_up"
    and "scale_down".
    """

    documentation = common_types.Uri
    "Documentation URI for the operation."

    target_resource = common_types.Uri
    "Target resource URI to the operation."

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/operations/resume',
                   name='resume',
                   type='operation',
                   tags=['small'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='A resume operation',
                   documentation='http://example.com/docs/resume_op',
                   target_resource='http://example.com/instances/uuid')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.controllers.camp.v1_1.datamodel import attribute_definitions
from solum.api.controllers.camp.v1_1.datamodel import type_definitions as model
from solum.api.controllers.camp.v1_1 import type_definitions
from solum import objects
from solum.tests import base
from solum.tests import fakes

TEST_ATTR_DEFINITION = (
    attribute_definitions.AttributeDefinition(type='attribute_definition',
                                              name='fake_attribute')
)

TEST_ATTR_DEF_LINK = (
    model.AttributeLink(href='fake_attribute_definition_url',
                        target_name='fake_attribute'))

TEST_TYPE_DEFINITION = (
    model.TypeDefinition(type='type_definition',
                         name='fake_type',
                         description='fake CAMP type definition',
                         attribute_definition_links=[TEST_ATTR_DEF_LINK])
)


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.camp.type_definition_handler.'
            'TypeDefinitionHandler')
@mock.patch('solum.api.handlers.camp.attribute_definition_handler.'
            'AttributeDefinitionHandler')
class TestTypeDefinitions(base.BaseTestCase):
    def setUp(self):
        super(TestTypeDefinitions, self).setUp()
        objects.load()

    def test_type_definitions_get(self,
                                  AttributeDefinitionHandler,
                                  TypeDefinitionHandler,
                                  resp_mock,
                                  request_mock):
        thand_get_all = TypeDefinitionHandler.return_value.get_all
        thand_get_all.return_value = [TEST_TYPE_DEFINITION]

        cont = type_definitions.TypeDefinitionsController()
        resp = cont.get()
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp['result'].type_definition_links)
        td_links = resp['result'].type_definition_links
        self.assertEqual(1, len(td_links))
        self.assertEqual(TEST_TYPE_DEFINITION.name, td_links[0].target_name)

    def test_type_definitions_get_one(self,
                                      AttributeDefinitionHandler,
                                      TypeDefinitionHandler,
                                      resp_mock,
                                      request_mock):
        ahand_get = AttributeDefinitionHandler.return_value.get
        ahand_get.return_value = TEST_ATTR_DEFINITION
        thand_get = TypeDefinitionHandler.return_value.get
        thand_get.return_value = TEST_TYPE_DEFINITION

        cont = type_definitions.TypeDefinitionsController()
        resp = cont.get_one('fake_type')
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertEqual(TEST_TYPE_DEFINITION.type, resp['result'].type)
        self.assertEqual(TEST_TYPE_DEFINITION.name, resp['result'].name)
        ad_links = resp['result'].attribute_definition_links
        self.assertEqual(1, len(ad_links))
        self.assertEqual(TEST_ATTR_DEFINITION.name,
                         ad_links[0].target_name)

    def test_type_definition_get_not_found(self,
                                           AttributeDefinitionHandler,
                                           TypeDefinitionHandler,
                                           resp_mock,
                                           request_mock):
        thand_get = TypeDefinitionHandler.return_value.get
        thand_get.return_value = None

        cont = type_definitions.TypeDefinitionsController()
        resp = cont.get_one('non_existent_type')
        self.assertIsNotNone(resp)
        self.assertEqual(404, resp_mock.status)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1.datamodel import extension as model
from solum.api.controllers.v1 import extension as controller
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.controllers.v1.extension.extension_handler.'
            'ExtensionHandler')
class TestExtensionController(base.BaseTestCase):
    def setUp(self):
        super(TestExtensionController, self).setUp()
        objects.load()

    def test_extension_get(self, handler_mock, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        handler_get = handler_mock.return_value.get
        fake_extension = fakes.FakeExtension()
        handler_get.return_value = fake_extension
        obj = controller.ExtensionController('test_id')
        result = obj.get()
        self.assertIsNotNone(result)
        self.assertEqual(fake_extension.name, result['result'].name)
        self.assertEqual(fake_extension.documentation,
                         result['result'].documentation)
        self.assertEqual(fake_extension.description,
                         result['result'].description)
        self.assertEqual(fake_extension.project_id,
                         result['result'].project_id)
        self.assertEqual(fake_extension.uuid, result['result'].uuid)
        self.assertEqual(fake_extension.version, result['result'].version)
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(result)
        handler_get.assert_called_once_with('test_id')

    def test_extension_get_not_found(self, handler_mock, resp_mock,
                                     request_mock, mock_policy):
        mock_policy.return_value = True
        handler_get = handler_mock.return_value.get
        handler_get.side_effect = exception.ResourceNotFound(
            name='extension', extension_id='test_id')
        obj = controller.ExtensionController('test_id')
        obj.get()
        self.assertEqual(404, resp_mock.status)
        handler_get.assert_called_once_with('test_id')

    def test_extension_put(self, handler_mock, resp_mock,
                           request_mock, mock_policy):
        json_update = {'description': 'foo_updated',
                       'user_id': 'user_id_changed',
                       'project_id': 'project_id_changed',
                       'version': '12.1',
                       'name': 'changed'}
        mock_policy.return_value = True
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        handler_update = handler_mock.return_value.update
        handler_update.return_value = fakes.FakeExtension()
        obj = controller.ExtensionController('test_id')
        obj.put(fakes.FakeExtension())
        self.assertEqual(200, resp_mock.status)
        handler_update.assert_called_once_with('test_id', json_update)

    def test_extension_put_none(self, handler_mock, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = None
        request_mock.content_type = 'application/json'
        handler_put = handler_mock.return_value.put
        handler_put.return_value = fakes.FakeExtension()
        controller.ExtensionController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_extension_put_not_found(self, handler_mock, resp_mock,
                                     request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'test_not_found'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        handler_update = handler_mock.return_value.update
        handler_update.side_effect = exception.ResourceNotFound(
            name='extension', extension_id='test_id')
        controller.ExtensionController('test_id').put()
        handler_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_extension_delete(self, mock_handler, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        handler_delete = mock_handler.return_value.delete
        handler_delete.return_value = None
        obj = controller.ExtensionController('test_id')
        obj.delete()
        handler_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)

    def test_extension_delete_not_found(self, mock_handler, resp_mock,
                                        request_mock, mock_policy):
        mock_policy.return_value = True
        handler_delete = mock_handler.return_value.delete
        handler_delete.side_effect = exception.ResourceNotFound(
            name='extension', extension_id='test_id')
        obj = controller.ExtensionController('test_id')
        obj.delete()
        handler_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.controllers.v1.extension.extension_handler.'
            'ExtensionHandler')
class TestExtensionsController(base.BaseTestCase):
    def setUp(self):
        super(TestExtensionsController, self).setUp()
        objects.load()

    def test_extensions_get_all(self, handler_mock, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get_all = handler_mock.return_value.get_all
        fake_extension = fakes.FakeExtension()
        hand_get_all.return_value = [fake_extension]
        obj = controller.ExtensionsController()
        resp = obj.get_all()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_extension.name, resp['result'][0].name)
        self.assertEqual(fake_extension.documentation,
                         resp['result'][0].documentation)
        self.assertEqual(fake_extension.description,
                         resp['result'][0].description)
        self.assertEqual(fake_extension.project_id,
                         resp['result'][0].project_id)
        self.assertEqual(fake_extension.uuid, resp['result'][0].uuid)
        self.assertEqual(fake_extension.version, resp['result'][0].version)
        hand_get_all.assert_called_with()
        self.assertEqual(200, resp_mock.status)

    def test_extensions_post(self, handler_mock, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo',
                       'description': 'foofoo',
                       'user_id': 'user_id_test',
                       'project_id': 'project_id_test',
                       'version': '1.3'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeExtension()
        obj = controller.ExtensionsController()
        obj.post(fakes.FakeExtension())
        self.assertEqual(201, resp_mock.status)
        handler_create.assert_called_once_with(json_update)

    def test_extensions_post_nodata(self, handler_mock,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeExtension()
        ret_val = controller.ExtensionsController().post()
        self.assertEqual("Missing argument: \"data\"",
                         str(ret_val['faultstring']))
        self.assertEqual(400, resp_mock.status)


class TestExtensionAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('empty', dict(data={})),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'name': 'foo',
                            'description': 'foofoo',
                            'user_id': 'user_id_test',
                            'project_id': 'project_id_test',
                            'version': '1.3'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = model.Extension()
            self.data = {}
        else:
            s = model.Extension(**self.data)
        self.data.pop('uri', None)
        self.data.pop('type', None)
        self.assertEqual(self.data, s.as_dict(objects.registry.Extension))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.handlers import pipeline_handler
from solum.common import catalog
from solum.common import exception
from solum.common import yamlutils
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils

from oslo_utils import uuidutils


@mock.patch('solum.objects.registry')
class TestPipelineHandler(base.BaseTestCase):
    def setUp(self):
        super(TestPipelineHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_pipeline_get(self, mock_registry):
        mock_registry.return_value.Pipeline.get_by_uuid.return_value = {
            'plan_id': '1234'
        }
        handler = pipeline_handler.PipelineHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        mock_registry.Pipeline.get_by_uuid.assert_called_once_with(
            self.ctx, 'test_id')

    def test_pipeline_get_all(self, mock_registry):
        mock_registry.PipelineList.get_all.return_value = {}
        handler = pipeline_handler.PipelineHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.PipelineList.get_all.assert_called_once_with(self.ctx)

    def test_update(self, mock_registry):
        data = {'user_id': 'new_user_id',
                'plan_uuid': 'input_plan_uuid'}
        handler = pipeline_handler.PipelineHandler(self.ctx)
        handler.update('test_id', data)
        mock_registry.Pipeline.update_and_save.assert_called_once_with(
            self.ctx, 'test_id', data)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_create(self, mock_kc, mock_registry):
        data = {'user_id': 'new_user_id',
                'uuid': 'input_uuid',
                'plan_uuid': 'input_plan_uuid'}

        db_obj = fakes.FakePipeline()
        mock_registry.Pipeline.return_value = db_obj
        fp = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = fp
        fp.raw_content = {
            'name': 'theplan',
            'artifacts': [{'name': 'nodeus',
                           'artifact_type': 'application.heroku',
                           'content': {
                               'href': 'https://example.com/ex.git'},
                           'language_pack': 'auto'}]}
        trust_ctx = utils.dummy_context()
        trust_ctx.trust_id = '12345'
        mock_kc.return_value.create_trust_context.return_value = trust_ctx

        handler = pipeline_handler.PipelineHandler(self.ctx)
        handler._execute_workbook = mock.MagicMock()
        handler._ensure_workbook = mock.MagicMock()
        res = handler.create(data)
        db_obj.update.assert_called_once_with(data)
        handler._execute_workbook.assert_called_once_with(db_obj)
        handler._ensure_workbook.assert_called_once_with(db_obj)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)
        mock_kc.return_value.create_trust_context.assert_called_once_with()

    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.common.catalog.get')
    def test_empty_create_stack(self, mock_get, mock_clients, mock_registry):
        db_obj = fakes.FakePipeline()
        test_name = uuidutils.generate_uuid()
        test_id = uuidutils.generate_uuid()
        db_obj.name = test_name
        mock_registry.Pipeline.return_value = db_obj

        fake_template = json.dumps({})
        mock_get.return_value = fake_template

        mock_create = mock_clients.return_value.heat.return_value.stacks.create
        mock_create.return_value = {"stack": {"id": test_id,
                                    "links": [{"href": "http://fake.ref",
                                               "rel": "self"}]}}
        handler = pipeline_handler.PipelineHandler(self.ctx)
        res = handler._create_empty_stack(db_obj)
        mock_create.assert_called_once_with(stack_name=test_name,
                                            template=fake_template)
        self.assertEqual(test_id, res)

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_delete(self, mock_clients, mock_registry):
        db_obj = fakes.FakePipeline()
        mock_exec = mock.MagicMock()
        mock_exec.context = json.dumps({'stack_id': 'foo'})
        mock_mistral = mock_clients.return_value.mistral.return_value
        mock_mistral.executions.get.return_value = mock_exec
        wb = yamlutils.dump({'Workflow': {
            'tasks': {'start':
                      {'parameters': {'stack_id': ''}}}}})
        mock_mistral.workbooks.get_definition.return_value = wb

        mock_registry.Pipeline.get_by_uuid.return_value = db_obj
        handler = pipeline_handler.PipelineHandler(self.ctx)
        handler.delete('test_id')
        db_obj.destroy.assert_called_once_with(self.ctx)
        mock_registry.Pipeline.get_by_uuid.assert_called_once_with(self.ctx,
                                                                   'test_id')

        mock_heat = mock_clients.return_value.heat.return_value
        mock_heat.stacks.delete.assert_called_once_with(stack_id='foo')

        mock_kc = mock_clients.return_value.keystone.return_value
        mock_kc.delete_trust.assert_called_once_with('trust_worthy')

    def test_trigger_workflow(self, mock_registry):
        trigger_id = 1
        db_obj = fakes.FakePipeline()
        mock_registry.Pipeline.get_by_trigger_id.return_value = db_obj
        plan_obj = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = plan_obj
        handler = pipeline_handler.PipelineHandler(self.ctx)
        handler._execute_workbook = mock.MagicMock()
        handler._context_from_trust_id = mock.MagicMock(return_value=self.ctx)
        handler.trigger_workflow(trigger_id)
        handler._execute_workbook.assert_called_once_with(db_obj)
        handler._context_from_trust_id.assert_called_once_with('trust_worthy')
        mock_registry.Pipeline.get_by_trigger_id.assert_called_once_with(
            None, trigger_id)

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_context_from_last_execution(self, mock_clients,
                                             mock_registry):
        fpipe = fakes.FakePipeline()
        mock_exec = mock.MagicMock()
        mock_exec.context = json.dumps({'stack_id': 'foo',
                                        'build_service_url': 'url-for',
                                        'base_image_id': '1-2-3-4',
                                        'source_format': 'heroku'})
        mock_mistral = mock_clients.return_value.mistral.return_value
        mock_mistral.executions.get.return_value = mock_exec
        wbook = catalog.get('workbooks', 'build_deploy')
        mock_mistral.workbooks.get_definition.return_value = wbook

        handler = pipeline_handler.PipelineHandler(self.ctx)

        ex_ctx = handler._get_context_from_last_execution(fpipe)
        self.assertEqual('foo', ex_ctx['stack_id'])
        self.assertEqual('url-for', ex_ctx['build_service_url'])
        self.assertEqual('1-2-3-4', ex_ctx['base_image_id'])
        self.assertEqual('heroku', ex_ctx['source_format'])

    @mock.patch('solum.common.heat_utils.get_network_parameters')
    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_build_execution_context_first_run(self, mock_ks, mock_net,
                                               mock_registry):
        fpipe = fakes.FakePipeline()
        fplan = fakes.FakePlan()
        mock_net.return_value = {'public_net': 'fake-net-id'}
        mock_registry.Plan.get_by_id.return_value = fplan
        fplan.raw_content = {
            'name': 'theplan',
            'artifacts': [{'name': 'nodeus',
                           'artifact_type': 'heroku',
                           'content': {
                               'href': 'https://example.com/ex.git'},
                           'language_pack': '1-2-3-4'}]}

        handler = pipeline_handler.PipelineHandler(self.ctx)

        handler._get_context_from_last_execution = mock.MagicMock(
            return_value=None)

        handler._create_empty_stack = mock.MagicMock(return_value='foo')
        url_for = mock_ks.return_value.client.service_catalog.url_for
        url_for.return_value = 'url-for'
        ex_ctx = handler._build_execution_context(fpipe)
        self.assertEqual('foo', ex_ctx['stack_id'])
        self.assertEqual('url-for', ex_ctx['build_service_url'])
        self.assertEqual('1-2-3-4', ex_ctx['base_image_id'])
        self.assertEqual('heroku', ex_ctx['source_format'])
        self.assertEqual('faker', ex_ctx['parameters']['app_name'])
        self.assertEqual('fake-net-id', ex_ctx['parameters']['public_net'])

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_build_execution_context_next_run(self, mock_clients,
                                              mock_registry):
        fpipe = fakes.FakePipeline()
        fplan = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = fplan
        fplan.raw_content = {
            'name': 'theplan',
            'artifacts': [{'name': 'nodeus',
                           'artifact_type': 'heroku',
                           'content': {
                               'href': 'https://example.com/ex.git'},
                           'language_pack': '1-2-3-4'}]}

        mock_exec = mock.MagicMock()
        mock_exec.context = json.dumps(
            {'stack_id': 'foo',
             'build_service_url': 'url-for',
             'base_image_id': '1-2-3-4',
             'source_format': 'heroku',
             'parameters': {'app_name': 'fruit',
                            'public_net': 'nsa-2-0'}})
        mock_mistral = mock_clients.return_value.mistral.return_value
        mock_mistral.executions.get.return_value = mock_exec
        wbook = catalog.get('workbooks', 'build_deploy')
        mock_mistral.workbooks.get_definition.return_value = wbook

        handler = pipeline_handler.PipelineHandler(self.ctx)

        ex_ctx = handler._build_execution_context(fpipe)
        self.assertEqual('foo', ex_ctx['stack_id'])
        self.assertEqual('url-for', ex_ctx['build_service_url'])
        self.assertEqual('1-2-3-4', ex_ctx['base_image_id'])
        self.assertEqual('heroku', ex_ctx['source_format'])
        self.assertEqual('fruit', ex_ctx['parameters']['app_name'])
        self.assertEqual('nsa-2-0', ex_ctx['parameters']['public_net'])

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_ensure_workbook_exists(self, mock_clients, mock_registry):
        fpipe = fakes.FakePipeline()
        fpipe.workbook_name = 'build'
        mock_mistral = mock_clients.return_value.mistral.return_value
        wbook = mock.MagicMock()
        wbook.workbook_name = 'build'
        mock_mistral.workbooks.get.return_value = wbook
        handler = pipeline_handler.PipelineHandler(self.ctx)
        handler._ensure_workbook(fpipe)
        mock_mistral.workbooks.create.assert_has_calls([])

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_ensure_workbook_not_exists(self, mock_clients, mock_registry):
        fpipe = fakes.FakePipeline()
        fpipe.workbook_name = 'build'
        mock_mistral = mock_clients.return_value.mistral.return_value
        mock_mistral.workbooks.get.side_effect = ValueError(
            'Workbook not found')
        handler = pipeline_handler.PipelineHandler(self.ctx)
        handler._ensure_workbook(fpipe)

        mock_mistral.workbooks.create.assert_called_once_with(
            'build', 'solum generated workbook', ['solum', 'builtin'])
        wbook = catalog.get('workbooks', 'build')
        mock_mistral.workbooks.upload_definition.assert_called_once_with(
            'build', wbook)

    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.common.catalog.get')
    def test_ensure_workbook_unknown(self, mock_get,
                                     mock_clients, mock_registry):
        fpipe = fakes.FakePipeline()
        fpipe.workbook_name = 'we-dont-have-this'
        mock_mistral = mock_clients.return_value.mistral.return_value
        mock_mistral.workbooks.get.side_effect = ValueError(
            'Workbook not found')
        mock_get.side_effect = exception.ObjectNotFound(
            name='workbook', id='we-dont-have-this')
        handler = pipeline_handler.PipelineHandler(self.ctx)
        self.assertRaises(exception.ObjectNotFound,
                          handler._ensure_workbook, fpipe)



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.objects import registry
from solum.objects.sqlalchemy import execution
from solum.tests import base
from solum.tests import utils


class TestExecution(base.BaseTestCase):
    def setUp(self):
        super(TestExecution, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()

        self.data = [{'pipeline_id': 'bla',
                      'uuid': 'ce43e347f0b0422825245b3e5f140a81cef6e65b'}]
        utils.create_models_from_data(execution.Execution, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertIsNotNone(registry.Execution)
        self.assertIsNotNone(registry.ExecutionList)

    def test_get_all(self):
        lst = execution.ExecutionList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        pl = execution.Execution().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(pl, key))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from unittest import mock
import wsme
import wsmeext.pecan as wsme_pecan

from solum.common import exception
from solum.tests import base
from solum.tests import fakes


class TestException(Exception):

    def __init__(self, *args, **kwargs):
        self.code = kwargs.pop('code', None)
        Exception.__init__(self, *args, **kwargs)


class TestPostData(wsme.types.Base):

    foo = wsme.types.text


class TestController(object):

    @exception.wrap_wsme_controller_exception
    def post(self, data):
        return self.mock_post(data)

    def mock_post(self, data):
        pass  # mock this


class TestPecanController(pecan.rest.RestController):

    @exception.wrap_pecan_controller_exception
    @pecan.expose()
    def post(self):
        return self.mock_post()

    def mock_post(self):
        pass  # mock this


class TestWsmePecanController(pecan.rest.RestController):

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(TestPostData, body=TestPostData, status=201)
    def post(self, data):
        return self.mock_post(data)

    def mock_post(self, data):
        # mock this
        pass


@mock.patch.object(TestController, 'mock_post')
class TestWsmeExceptionWrapper(base.BaseTestCase):

    def test_wrap_500(self, mockpost):
        mockpost.__name__ = 'post'
        thrown = (TestException(code=500),
                  TestException(code=700),
                  TestException(code=1450),
                  Exception("This is an error"))
        mockpost.side_effect = [e for e in thrown]
        controller = TestController()
        for expect in thrown:
            exc = self.assertRaises(wsme.exc.ClientSideError, controller.post,
                                    'foo')
            self.assertIn('Your request could not be handled because of a '
                          'problem in the server.', str(exc))
            self.assertEqual(expect.code if hasattr(expect, 'code') else 500,
                             exc.code)

    def test_wrap_under_500(self, mockpost):
        mockpost.__name__ = 'post'
        thrown = (TestException("Test Error 1", code=400),
                  TestException("Test Error 2", code=0))
        mockpost.side_effect = [e for e in thrown]
        controller = TestController()
        for expect in thrown:
            exc = self.assertRaises(wsme.exc.ClientSideError, controller.post,
                                    'foo')
            self.assertIn(str(expect), str(exc))
            self.assertEqual(expect.code, exc.code)


@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch.object(TestPecanController, 'mock_post', autospec=True)
class TestPecanExceptionWrapper(base.BaseTestCase):

    def test_wrap_500(self, mockpost, mockreq, mockresp):
        mockpost.__name__ = 'post'
        mockreq.__name__ = 'request'
        mockresp.__name__ = 'response'
        thrown = [TestException(code=500),
                  TestException(code=700),
                  TestException(code=1450),
                  Exception("This is an error")]
        mockpost.side_effect = (t for t in thrown)
        controller = TestPecanController()
        for expect in thrown:
            controller.post()
            self.assertIn('Your request could not be handled because of a '
                          'problem in the server.', mockresp.text)
            self.assertEqual(expect.code if hasattr(expect, "code") else 500,
                             mockresp.status)

    def test_wrap_under_500(self, mockpost, mockreq, mockresp):
        mockpost.__name__ = 'post'
        mockreq.__name__ = 'request'
        mockresp.__name__ = 'response'
        thrown = (TestException("Test Error 1", code=400),
                  TestException("Test Error 2", code=0))
        mockpost.side_effect = (t for t in thrown)
        controller = TestPecanController()
        for exp in thrown:
            controller.post()
            self.assertIn(str(exp), mockresp.text)
            self.assertEqual(exp.code, mockresp.status)


@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch.object(TestWsmePecanController, 'mock_post', autospec=True)
class TestWsmePecanExceptionWrapper(base.BaseTestCase):

    def test_wrap_500(self, mockpost, mockreq, mockresp):
        mockpost.__name__ = 'post'
        mockreq.__name__ = 'request'
        mockresp.__name__ = 'response'
        mockresp.status_code = 500
        mockpost.side_effect = Exception("This is a hidden message")
        mockreq.body = '{"foo":"bar"}'
        mockreq.content_type = "application/json"
        resp = TestWsmePecanController().post()
        self.assertIn('Your request could not be handled because of a '
                      'problem in the server.', resp.get('faultstring'))

    def test_wrap_under_500(self, mockpost, mockreq, mockresp):
        mockpost.__name__ = 'post'
        mockreq.__name__ = 'request'
        mockresp.__name__ = 'response'
        mockpost.side_effect = exception.BadRequest(reason="Testing")
        mockreq.body = '{"foo":"bar"}'
        mockreq.content_type = "application/json"
        resp = TestWsmePecanController().post()
        self.assertEqual(exception.BadRequest.code, mockresp.status)
        self.assertIn('Testing', resp.get('faultstring'))



# Copyright (c) 2015 Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.handlers import app_handler
from solum.common import exception as exc
from solum import objects
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry')
class TestAppHandler(base.BaseTestCase):
    def setUp(self):
        super(TestAppHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_app_get(self, mock_registry):
        mock_registry.App.get_by_uuid.return_value = {}
        handler = app_handler.AppHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        get_by_uuid = mock_registry.App.get_by_uuid
        get_by_uuid.assert_called_once_with(self.ctx, 'test_id')

    def test_app_get_all(self, mock_registry):
        mock_registry.App.get_all.return_value = {}
        handler = app_handler.AppHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.AppList.get_all.assert_called_once_with(self.ctx)

    @mock.patch('solum.deployer.api.API.destroy_app')
    def test_app_delete(self, mock_destroy, mock_registry):
        handler = app_handler.AppHandler(self.ctx)
        handler.delete('test_id')
        mock_destroy.assert_called_once_with('test_id')

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_app_create(self, mock_kc, mock_registry):
        data = {'name': 'fakeapp',
                'description': 'fake app for testing',
                'source': {
                    'repository': 'https://github.com/example/a.git',
                    'revision': 'master'
                }}
        raw_content = {'a': 'b'}
        data['raw_content'] = json.dumps(raw_content)
        self.ctx.password = 'password'
        db_obj = fakes.FakeApp()
        mock_registry.App.return_value = db_obj
        handler = app_handler.AppHandler(self.ctx)
        res = handler.create(data)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_app_create_invalid_repo_url(self, mock_kc, mock_registry):
        invalid_urls_list = list()
        invalid_urls_list.append('http://github.com/skdhfskjhdks')
        invalid_urls_list.append('github.com/abc/xyz')
        invalid_urls_list.append('xyz://github.com/abc/xyz.git')
        invalid_urls_list.append('xyz://github.com/abc/xyz')
        invalid_urls_list.append('abc')
        invalid_urls_list.append('http')
        invalid_urls_list.append('git')

        for invalid_url in invalid_urls_list:
            data = {'name': 'fakeapp',
                    'description': 'fake app for testing',
                    'source': {
                        'repository': invalid_url,
                        'revision': 'master'
                    }}
            raw_content = {'a': 'b'}
            data['raw_content'] = json.dumps(raw_content)
            self.ctx.password = 'password'

            db_obj = fakes.FakeApp()
            mock_registry.App.return_value = db_obj
            handler = app_handler.AppHandler(self.ctx)

            self.assertRaises(exc.BadRequest, handler.create, data)
            assert not db_obj.create.called, 'db_obj.create called'

    def test_app_patch(self, mock_registry):
        mock_registry.App.side_effect = [mock.MagicMock(), mock.MagicMock()]
        db_obj = objects.registry.App()
        # Without this, I'd just get a mocked function call.
        # I want real data so I can track that it's being updated.
        a = fakes.FakeApp().as_dict()
        db_obj.as_dict.return_value = a
        mock_registry.App.get_by_uuid.return_value = db_obj
        # I'm not saving anything anyway, I just want to make sure
        # that I'm sending the right data to update_and_save.
        mock_registry.App.update_and_save = (
            lambda context, obj_id, data_dict: data_dict)

        # Build a phony app whose as_dict returns this dict below.
        to_update_data = {
            'name': 'newfakeapp',
            'workflow_config': {
                'run_cmd': 'newruncommand',
            },
            'source': {
                'revision': 'experimental',
            },
            'repo_token': 'abc'
        }
        handler = app_handler.AppHandler(self.ctx)
        new_app = objects.registry.App()
        new_app.as_dict.return_value = to_update_data

        # The actual call to handler.patch
        updated = handler.patch('test_id', new_app)

        self.assertEqual('newfakeapp', updated['name'])
        self.assertEqual('newruncommand',
                         updated['workflow_config']['run_cmd'])
        self.assertEqual('python ./tests.py',
                         updated['workflow_config']['test_cmd'])
        self.assertEqual('http://github.com/example/a.git',
                         updated['source']['repository'])
        self.assertEqual('experimental',
                         updated['source']['revision'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.handlers import operation_handler as operation
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry')
class TestOperationHandler(base.BaseTestCase):

    def setUp(self):
        super(TestOperationHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_get(self, mock_registry):
        mock_registry.Operation.get_by_uuid.return_value = {}
        handler = operation.OperationHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        mock_registry.Operation.get_by_uuid.assert_called_once_with(self.ctx,
                                                                    'test_id')

    def test_get_all(self, mock_registry):
        mock_registry.OperationList.get_all.return_value = {}
        handler = operation.OperationHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.OperationList.get_all.assert_called_once_with(self.ctx)

    def test_update(self, mock_registry):
        data = {'user_id': 'new_user_id'}
        handler = operation.OperationHandler(self.ctx)
        handler.update('test_id', data)
        mock_registry.Operation.update_and_save.assert_called_once_with(
            self.ctx, 'test_id', data)

    def test_create(self, mock_registry):
        data = {'user_id': 'new_user_id'}
        db_obj = fakes.FakeOperation()
        mock_registry.Operation.return_value = db_obj
        handler = operation.OperationHandler(self.ctx)
        res = handler.create(data)
        db_obj.update.assert_called_once_with(data)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)

    def test_delete(self, mock_registry):
        db_obj = fakes.FakeOperation()
        mock_registry.Operation.get_by_uuid.return_value = db_obj
        handler = operation.OperationHandler(self.ctx)
        handler.delete('test_id')
        db_obj.destroy.assert_called_once_with(self.ctx)
        mock_registry.Operation.get_by_uuid.assert_called_once_with(self.ctx,
                                                                    'test_id')



# Copyright 2013 - Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum specific config handling."""

from oslo_config import cfg

from solum.common import config as common_config
from solum import version


def parse_args(argv, default_config_files=None):
    cfg.CONF(argv[1:],
             project='solum',
             version=version.version_string(),
             default_config_files=default_config_files)
    common_config.set_config_defaults()



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import plan as solum_plan
from solum.api.controllers.v1.datamodel import types as api_types


class ServiceSpecification(solum_plan.ServiceReference):
    """CAMP v1.1 ServiceSpecification."""

    description = wtypes.text
    """A description of the specified service."""

    tags = [wtypes.text]
    """Tags for the specified service."""

    href = wtypes.text
    """Reference to a service resource that resolves the service described by
    this ServiceSpecification."""

    characteristics = [wtypes.DictType(wtypes.text, wtypes.text)]
    """An array of dictionaries that express the desired characteristics of
    any service that matches this specification."""

    def __init__(self, **kwargs):
        super(ServiceSpecification, self).__init__(**kwargs)


class RequirementSpecification(solum_plan.Requirement):
    """CAMP v1.1 RequirementSpecification."""

    fulfillment = api_types.MultiType(wtypes.text,
                                      ServiceSpecification)
    """Either a ServiceSpecification or a reference to a
    ServiceSpecification."""

    def __init__(self, **kwargs):
        if 'fulfillment' in kwargs:
            fulfill = kwargs.get('fulfillment')
            if type(fulfill) is not str:
                kwargs['fulfillment'] = ServiceSpecification(**fulfill)

        super(RequirementSpecification, self).__init__(**kwargs)


class ArtifactSpecification(solum_plan.Artifact):
    """CAMP v1.1 ArtifactSpecification."""

    description = wtypes.text
    """A description of the specified artifact."""

    tags = [wtypes.text]
    """Tags for the specified artifact."""

    requirements = [RequirementSpecification]
    """List of CAMP Requirement Specifications for the artifact."""

    def __init__(self, **kwargs):
        if 'requirements' in kwargs:
            kwargs['requirements'] = [RequirementSpecification(**rs)
                                      for rs in kwargs.get('requirements', [])]

        # Call our grandparent's (Base) constructor since we have already done
        # the work (creating objects from mappings) of our parent's (Artifact)
        # constructor and, if called, our parent's constructor will generate
        # an exception when it tries to create objects from these objects.
        wtypes.Base.__init__(self, **kwargs)


class Plan(solum_plan.Plan):
    """CAMP v1.1 plan resource."""

    camp_version = wsme.types.wsattr(wtypes.text, mandatory=True)
    """The value of this attribute expresses the version of the CAMP
    specification to which the Plan conforms."""

    origin = wtypes.text
    """The value of this attribute specifies the origin of the Plan."""

    artifacts = [ArtifactSpecification]
    """List of CAMP Artifact Specifications (see Section 4.3.3)."""

    services = [ServiceSpecification]
    """List of CAMP Service Specifications (see Section 4.3.6)."""

    def __init__(self, **kwargs):
        if 'artifacts' in kwargs:
            kwargs['artifacts'] = [ArtifactSpecification(**art)
                                   for art in kwargs.get('artifacts', [])]
        if 'services' in kwargs:
            kwargs['services'] = [ServiceSpecification(**sr)
                                  for sr in kwargs.get('services', [])]

        # Call our grandparent's (Base) constructor since we have already done
        # the work (creating objects from mappings) of our parent's (Plan)
        # constructor and, if called, our parent's constructor will generate
        # an exception when it tries to create objects from these objects.
        api_types.Base.__init__(self, **kwargs)


class Plans(api_types.Base):
    """CAMP v1.1 plans resource."""

    plan_links = [common_types.Link]
    """This attribute contains Links to the plan resources that represent the
    blueprints for applications deployed on the platform."""

    parameter_definitions_uri = common_types.Uri
    """The value of the parameter_definitions_uri attribute references a
    resource that contains links to parameter_definition resources that
    describe the parameters accepted by this resource on an HTTP POST
    method."""

    def __init__(self, **kwds):
        super(Plans, self).__init__(**kwds)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class CrudMixin(object):

    @classmethod
    def get_by_id(cls, context, id):
        """Return a specific object by its unique identifier."""

    @classmethod
    def get_by_uuid(cls, context, item_uuid):
        """Return a specific object by its uuid."""

    def create(self, context):
        """Create the model."""

    def save(self, context):
        """Change the model."""

    def destroy(self, context):
        """Destroy the model."""

    def add_forward_schema_changes(self):
        """Update the attributes of self to include new schema elements.

        This method is invoked during save/create operations to guarantee
        that objects being created during a schema transition support both
        new and old schemas.  This ensures that background jobs can run
        to migrate existing objects on a live system while writes are
        occurring.

        The changes made by this method must match the schema defined
        for "transitioning" and "new" (see solum.objects.__init__).
        """


class CrudListMixin(object):
    @classmethod
    def get_all(cls, context):
        """Retrieve all applications for the active context.

        Context may be global or tenant scoped.
        """



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import service
from solum.api.handlers import service_handler
from solum.common import exception
from solum.common import policy
from solum import objects


class ServiceController(rest.RestController):
    """Manages operations on a single service."""

    def __init__(self, service_id):
        super(ServiceController, self).__init__()
        self._id = service_id

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(service.Service)
    def get(self):
        """Return this service."""
        policy.check('show_service',
                     pecan.request.security_context)
        handler = service_handler.ServiceHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return service.Service.from_db_model(handler.get(self._id), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(service.Service, body=service.Service)
    def put(self, data):
        """Modify this service."""
        policy.check('update_service',
                     pecan.request.security_context)
        handler = service_handler.ServiceHandler(
            pecan.request.security_context)
        res = handler.update(self._id,
                             data.as_dict(objects.registry.Service))
        host_url = pecan.request.application_url.rstrip('/')
        return service.Service.from_db_model(res, host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=204)
    def delete(self):
        """Delete this service."""
        policy.check('delete_service',
                     pecan.request.security_context)
        handler = service_handler.ServiceHandler(
            pecan.request.security_context)
        return handler.delete(self._id)


class ServicesController(rest.RestController):
    """Manages operations on the services collection."""

    @pecan.expose()
    def _lookup(self, service_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return ServiceController(service_id), remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(service.Service, body=service.Service,
                         status_code=201)
    def post(self, data):
        """Create a new service."""
        policy.check('create_service',
                     pecan.request.security_context)
        handler = service_handler.ServiceHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return service.Service.from_db_model(
            handler.create(data.as_dict(objects.registry.Service)), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([service.Service])
    def get_all(self):
        """Return all services, based on the query provided."""
        policy.check('get_services',
                     pecan.request.security_context)
        handler = service_handler.ServiceHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [service.Service.from_db_model(ser, host_url)
                for ser in handler.get_all()]



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils
from swiftclient import exceptions as swiftexp

from solum.api.handlers import common
from solum.api.handlers import handler
from solum.api.handlers import userlog_handler
from solum.common import exception as exc
from solum.common import solum_glanceclient
from solum.common import solum_swiftclient
from solum import objects
from solum.objects import image
from solum.objects.sqlalchemy import app
from solum.worker import api


LOG = logging.getLogger(__name__)

# Register options for the service
API_SERVICE_OPTS = [
    cfg.StrOpt('operator_project_id',
               default='',
               help='Tenant id of the operator account used to create LPs'),
    cfg.IntOpt('max_languagepack_limit',
               default=10,
               help='Maximum number of languagepacks allowed'),
]


def list_opts():
    yield 'api', API_SERVICE_OPTS


CONF = cfg.CONF
CONF.register_opts(API_SERVICE_OPTS, group='api')
cfg.CONF.import_opt('image_storage', 'solum.worker.config', group='worker')


class LanguagePackHandler(handler.Handler):
    """Fulfills a request on the languagepack resource."""

    def _check_lp_referenced_in_any_plan(self, ctxt, db_obj):
        plans = objects.registry.PlanList.get_all(ctxt)
        for plan in plans:
            # (devkulkarni): Adding check for raw_contents because
            # there are 'plans' in our db that don't have the
            # raw_contents field. These plans correspond to the
            # 'dummy' plans that we are creating corresponding
            # to 'app deploy' requests.
            if hasattr(plan, 'raw_content') and plan.raw_content:
                if plan.raw_content and 'artifacts' in plan.raw_content:
                    for artifact in plan.raw_content['artifacts']:
                        lp = artifact.get('language_pack', '')
                        if lp == db_obj.name or lp == db_obj.uuid:
                            return True

    def _check_lp_referenced_in_any_app(self, ctxt, db_obj):
        apps = app.App.get_all_by_lp(ctxt, db_obj.name)
        if len(apps) > 0:
            return True
        else:
            apps = app.App.get_all_by_lp(ctxt, db_obj.uuid)
            if len(apps) > 0:
                return True
            else:
                return False

    def _check_if_limit_of_lps_reached(self, ctxt):
        num_of_lps = objects.registry.Image.get_num_of_lps(ctxt)
        if num_of_lps >= int(cfg.CONF.api.max_languagepack_limit):
            msg = "Number of languagepacks reached limit of '%s'." % (
                cfg.CONF.api.max_languagepack_limit)
            raise exc.ResourceLimitExceeded(reason=msg)

    def get(self, id):
        """Return a languagepack."""
        return objects.registry.Image.get_lp_by_name_or_uuid(
            self.context, id, include_operators_lp=True)

    def get_all(self):
        """Return all languagepacks."""
        return objects.registry.Image.get_all_languagepacks(self.context)

    def create(self, data, lp_metadata, lp_params):
        """Create a new languagepack."""

        self._check_if_limit_of_lps_reached(self.context)

        common.check_url(data['source_uri'])

        try:
            # Check if an LP with the same name exists.
            objects.registry.Image.get_lp_by_name_or_uuid(
                self.context, data['name'], include_operators_lp=True)
        except exc.ResourceNotFound:
            db_obj = objects.registry.Image()
            db_obj.update(data)
            db_obj.uuid = uuidutils.generate_uuid()
            db_obj.user_id = self.context.user
            db_obj.project_id = self.context.project_id
            db_obj.status = image.States.QUEUED
            db_obj.artifact_type = 'language_pack'
            if lp_metadata:
                db_obj.tags = []
                db_obj.tags.append(lp_metadata)

            db_obj.create(self.context)
            self._start_build(db_obj, lp_params)
            return db_obj
        else:
            raise exc.ResourceExists(name=data['name'])

    def delete(self, uuid):
        """Delete a languagepack."""
        db_obj = objects.registry.Image.get_lp_by_name_or_uuid(self.context,
                                                               uuid)
        # Check if the languagepack is being used.
        if (self._check_lp_referenced_in_any_plan(self.context, db_obj) or
                self._check_lp_referenced_in_any_app(self.context, db_obj)):
            raise exc.LPStillReferenced(name=uuid)

        if db_obj.docker_image_name:
            img_filename = db_obj.docker_image_name.split('-', 1)[1]

            if cfg.CONF.worker.image_storage == 'swift':
                # Delete image file from swift
                try:
                    swift = solum_swiftclient.SwiftClient(self.context)
                    swift.delete_object('solum_lp', img_filename)
                except swiftexp.ClientException:
                    raise exc.AuthorizationFailure(
                        client='swift',
                        message="Unable to delete languagepack image "
                                "from swift.")
            elif cfg.CONF.worker.image_storage == 'glance':
                glance = solum_glanceclient.GlanceClient(self.context)
                glance.delete_image_by_name(img_filename)
            else:
                LOG.error("Unrecognized image_storage option specified.")
                return

        # Delete logs
        log_handler = userlog_handler.UserlogHandler(self.context)
        log_handler.delete(db_obj.uuid)

        return db_obj.destroy(self.context)

    def _start_build(self, image, lp_params):
        git_info = {
            'source_url': image.source_uri,
        }
        api.API(context=self.context).build_lp(
            image_id=image.id,
            git_info=git_info,
            name=image.name,
            source_format=image.source_format,
            image_format=image.image_format,
            artifact_type=image.artifact_type,
            lp_params=lp_params)



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers.v1.datamodel import types as api_types


class Service(api_types.Base):
    """The Service resource represents a networked service.

    You may create Component resources that refer to
    Service resources. A Component represents an instance of a Service.
    Your application connects to such a Component using a network protocol.
    For example, the Platform may offer a default Service named "mysql".
    You may create multiple Component resources that reference different
    instances of the "mysql" service. Each Component may be a multi-tenant
    instance of a MySQL database (perhaps a logical database) service offered
    by the Platform for a given Assembly.
    """

    read_only = bool
    """The service is read only when this value is true."""

    service_type = wtypes.text
    """Type of service. Example: language_pack or db::mysql"""

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/language_packs/java1.4',
                   name='language-pack',
                   type='service',
                   service_type='language_pack',
                   description='A language pack service',
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   tags=['group_xyz'],
                   read_only=False)

    @classmethod
    def sample1(cls):
        return cls(uri='http://example.com/v1/database/mysql',
                   name='database',
                   type='service',
                   service_type='db::mysql',
                   description='A mysql service',
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   tags=['group_xyz'],
                   read_only=False)



# Copyright 2015 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1 import app
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.app_handler.AppHandler')
class TestAppController(base.BaseTestCase):
    def setUp(self):
        super(TestAppController, self).setUp()
        objects.load()

    def test_app_get(self, AppHandler, resp_mock, request_mock):
        fake_app = fakes.FakeApp()
        hand_get = AppHandler.return_value.get
        hand_get.return_value = fake_app
        cont = app.AppController('test_id')
        resp = cont.get()
        self.assertIsNotNone(resp)
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_app_get_notfound(self, AppHandler, resp_mock, request_mock):
        hand_get = AppHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(name='app',
                                                          id='test_id')
        app.AppController('test_id').get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_app_patch(self, AppHandler, resp_mock, request_mock):
        json_update = {
            'name': 'newfakeappname',
            'workflow_config': {
                'run_cmd': 'newruncmd',
            },
            'source': {
                'repository': 'newrepo',
            },
        }
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        hand_patch = AppHandler.return_value.patch
        fake_app = objects.registry.App(**json_update)
        hand_patch.return_value = fake_app
        app.AppController('test_id').patch()
        hand_patch.assert_called_once_with('test_id', mock.ANY)
        updated_app = hand_patch.call_args[0][1]
        self.assertEqual('newfakeappname', updated_app.name)
        self.assertEqual('newruncmd', updated_app.workflow_config['run_cmd'])
        self.assertEqual('newrepo', updated_app.source['repository'])
        self.assertEqual(200, resp_mock.status)

    def test_app_patch_no_data(self, AppHandler, resp_mock, request_mock):
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        app.AppController('test_id').patch()
        self.assertEqual(400, resp_mock.status)

    def test_app_delete(self, AppHandler, resp_mock, request_mock):
        hand_delete = AppHandler.return_value.delete
        hand_delete.return_value = None
        obj = app.AppController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(202, resp_mock.status)

    def test_app_delete_notfound(self, AppHandler, resp_mock, request_mock):
        hand_delete = AppHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            name='missing_app', app_id='test_id')
        obj = app.AppController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)


@mock.patch('solum.objects.registry.Image')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.app_handler.AppHandler')
class TestAppsController(base.BaseTestCase):
    def setUp(self):
        super(TestAppsController, self).setUp()
        objects.load()

    def test_apps_post(self, AppHandler, resp_mock, request_mock, mock_img):
        json_create = {'name': 'fakeapp',
                       'languagepack': 'fakelp'}
        mock_img.get_lp_by_name_or_uuid.return_value = {}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        hand_create = AppHandler.return_value.create
        fake_app = objects.registry.App(**json_create)
        hand_create.return_value = fake_app
        app.AppsController().post()
        self.assertTrue(hand_create.called)
        created_app = hand_create.call_args[0][0]
        self.assertEqual(created_app['name'], json_create['name'])
        self.assertEqual(201, resp_mock.status)

    def test_apps_post_lp_not_found(self, AppHandler, resp_mock, request_mock,
                                    mock_img):
        json_create = {'name': 'fakeapp',
                       'languagepack': 'fakelp'}
        fi = fakes.FakeImage()
        not_found = exception.ResourceNotFound(name='lp', id='fakelp')
        mock_img.get_lp_by_name_or_uuid.side_effect = not_found
        mock_img.return_value = fi
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        hand_create = AppHandler.return_value.create
        fake_app = objects.registry.App(**json_create)
        hand_create.return_value = fake_app
        ret = app.AppsController().post()
        expected = 'The Languagepack fakelp could not be found.'
        self.assertEqual(expected, ret['faultstring'])

    def test_apps_post_no_data(self, AppHandler, resp_mock, request_mock,
                               mock_img):
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        app.AppsController().post()
        self.assertEqual(400, resp_mock.status)

    def test_apps_post_no_app_name(self, AppHandler, resp_mock, request_mock,
                                   mock_img):
        json_create = {'name': '',
                       'languagepack': 'fakelp'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        hand_create = AppHandler.return_value.create
        fake_app = objects.registry.App(**json_create)
        hand_create.return_value = fake_app
        app.AppsController().post()
        self.assertEqual(400, resp_mock.status)

    def test_apps_get_all(self, AppHandler, resp_mock, request_mock,
                          mock_img):
        hand_get = AppHandler.return_value.get_all
        fake_app = fakes.FakeApp()
        hand_get.return_value = [fake_app]
        resp = app.AppsController().get_all()
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        hand_get.assert_called_with()



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class PlatformEndpoint(api_types.Base):
    """CAMP v1.1 platform_endpoint resource model."""

    platform_uri = common_types.Uri
    """The URI of the platform resource that this platform_endpoint resource
       describes."""

    specification_version = wtypes.text
    """Specification Version String of the CAMP spec supported by the platform
       referenced by this resource."""

    backward_compatible_specification_versions = [wtypes.text]
    """Identifies each version of the CAMP spec that is backwards compatible
       with the spec supported by the platform referenced by this resource."""

    implementation_version = wtypes.text
    """Provider specific implementation version supported by the platform
       referenced by this resource."""

    backward_compatible_implementation_versions = [wtypes.text]
    """Identifies each implementation version that is backwards compatible with
       the implementation version of the platform referenced by this
       resource."""

    auth_scheme = wtypes.text
    """Indicates the authentication scheme/mechanism expected by the referenced
       platform resource."""

    def __init__(self, **kwds):
        super(PlatformEndpoint, self).__init__(**kwds)



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import os

from swiftclient import exceptions as swiftexp

from solum.api.handlers import handler
from solum.common import exception as exc
from solum.common import solum_swiftclient
from solum import objects


class UserlogHandler(handler.Handler):

    def get_all(self):
        """Return all userlogs, based on the query provided."""
        return objects.registry.UserlogList.get_all(self.context)

    def get_all_by_id(self, resource_uuid):
        return objects.registry.UserlogList.get_all_by_id(
            self.context, resource_uuid=resource_uuid)

    def delete(self, resource_uuid):
        """Delete existing logs."""
        ulogs = objects.registry.UserlogList.get_all_by_id(
            self.context, resource_uuid=resource_uuid)

        # Delete log files
        swift = solum_swiftclient.SwiftClient(self.context)
        for ulog in ulogs:
            location = ulog.location
            strategy = ulog.strategy
            strategy_info = json.loads(ulog.strategy_info)
            if strategy == 'swift':
                # Delete logs from swift
                try:
                    swift.delete_object(strategy_info['container'],
                                        location)
                except swiftexp.ClientException:
                    raise exc.AuthorizationFailure(
                        client='swift',
                        message="Unable to delete logs from swift.")
            elif strategy == 'local':
                # Delete logs from local filesystem
                # This setting is exclusively used for single node deployments.
                try:
                    os.remove(location)
                except OSError:
                    pass

            # Delete the log reference from db.
            ulog.destroy(self.context)

        return



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from pecan import core
from wsme import types as wtypes

from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers.v1.datamodel import types as api_types


class AttributeDefinition(api_types.Base):
    """attribute_definition resource

    An attribute_definition resource represents exactly one supported
    attribute of one or more resource types.
    """

    documentation = wtypes.text
    """The value of the documentation attribute is a URI that references the
     documentation for the attribute that this resource describes."""

    attribute_type = wtypes.text
    """The value of the attribute_type attribute specifies the type of the
     attribute that this resource describes."""

    @classmethod
    def from_json(cls, dct):
        ret_val = cls()
        for key, value in dct.items():
            if hasattr(ret_val, key):
                setattr(ret_val, key, value)
            else:
                core.abort(500, 'internal metadata is incorrect')
        return ret_val

    def fix_uris(self, host_url):
        """Update URIs to reflect a host URL."""
        ret_val = copy.deepcopy(self)
        ret_val.uri = uris.ATTRIBUTE_DEF_URI_STR % (host_url, ret_val.uri)
        return ret_val



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Starter script for the Solum Deployer service."""

import os
import sys

from oslo_config import cfg
from oslo_log import log as logging

from solum.common.rpc import service as rpc_service
from solum.common import service
from solum.deployer.handlers import heat as heat_handler
from solum.deployer.handlers import noop as noop_handler
from solum.i18n import _

LOG = logging.getLogger(__name__)


def main():
    service.prepare_service(sys.argv)

    LOG.info(_('Starting server in PID %s') % os.getpid())
    LOG.debug("Configuration:")
    logging.setup(cfg.CONF, 'solum')

    cfg.CONF.import_opt('topic', 'solum.deployer.config', group='deployer')
    cfg.CONF.import_opt('host', 'solum.deployer.config', group='deployer')
    cfg.CONF.import_opt('handler', 'solum.deployer.config', group='deployer')

    handlers = {
        'noop': noop_handler.Handler,
        'heat': heat_handler.Handler,
    }

    endpoints = [
        handlers[cfg.CONF.deployer.handler](),
    ]

    server = rpc_service.Service(cfg.CONF.deployer.topic,
                                 cfg.CONF.deployer.host, endpoints)
    server.serve()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1 import assembly
from solum.api.controllers.v1.datamodel import assembly as assemblymodel
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.assembly_handler.AssemblyHandler')
class TestAssemblyController(base.BaseTestCase):
    def setUp(self):
        super(TestAssemblyController, self).setUp()
        objects.load()

    def test_assembly_get(self, AssemblyHandler, resp_mock,
                          request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = AssemblyHandler.return_value.get
        fake_assembly = fakes.FakeAssembly()
        hand_get.return_value = fake_assembly
        cont = assembly.AssemblyController('test_id')
        resp = cont.get()
        self.assertIsNotNone(fake_assembly)
        self.assertEqual(fake_assembly.name, resp['result'].name)
        self.assertEqual(fake_assembly.project_id, resp['result'].project_id)
        self.assertEqual(fake_assembly.uuid, resp['result'].uuid)
        self.assertEqual(fake_assembly.status, resp['result'].status)
        self.assertEqual(fake_assembly.user_id, resp['result'].user_id)
        self.assertEqual(fake_assembly.application_uri,
                         resp['result'].application_uri)
        self.assertEqual(fake_assembly.created_at, resp['result'].created_at)
        self.assertEqual(fake_assembly.updated_at, resp['result'].updated_at)
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_assembly_get_not_found(self, AssemblyHandler,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = AssemblyHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(
            name='assembly', assembly_id='test_id')
        cont = assembly.AssemblyController('test_id')
        cont.get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_assembly_put_none(self, AssemblyHandler, resp_mock,
                               request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = None
        request_mock.content_type = 'application/json'
        hand_put = AssemblyHandler.return_value.put
        hand_put.return_value = fakes.FakeAssembly()
        assembly.AssemblyController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_assembly_put_not_found(self, AssemblyHandler,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = AssemblyHandler.return_value.update
        hand_update.side_effect = exception.ResourceNotFound(
            name='assembly', assembly_id='test_id')
        assembly.AssemblyController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_assembly_put_ok(self, AssemblyHandler, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = AssemblyHandler.return_value.update
        hand_update.return_value = fakes.FakeAssembly()
        assembly.AssemblyController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(200, resp_mock.status)

    def test_assembly_delete_not_found(self, AssemblyHandler,
                                       resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = AssemblyHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            name='assembly', assembly_id='test_id')
        obj = assembly.AssemblyController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_assembly_delete_ok(self, AssemblyHandler,
                                resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = AssemblyHandler.return_value.delete
        hand_delete.return_value = None
        obj = assembly.AssemblyController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)


class TestAssemblyAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'uri': 'http://example.com/v1/assemblies/x1',
                            'name': 'Example-assembly',
                            'type': 'assembly',
                            'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                            'user_id': '55f41cf46df74320b9486a35f5d28a11'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = assemblymodel.Assembly()
            self.data = {}
        else:
            s = assemblymodel.Assembly(**self.data)
        if 'uri' in self.data:
            del self.data['uri']
        if 'type' in self.data:
            del self.data['type']

        self.assertEqual(self.data, s.as_dict(objects.registry.Assembly))


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.assembly_handler.AssemblyHandler')
class TestAssembliesController(base.BaseTestCase):
    def setUp(self):
        super(TestAssembliesController, self).setUp()
        objects.load()

    def test_assemblies_get_all(self, AssemblyHandler,
                                resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = AssemblyHandler.return_value.get_all
        fake_assembly = fakes.FakeAssembly()
        hand_get.return_value = [fake_assembly]
        resp = assembly.AssembliesController().get_all()
        self.assertEqual(fake_assembly.name, resp['result'][0].name)
        self.assertEqual(fake_assembly.project_id,
                         resp['result'][0].project_id)
        self.assertEqual(fake_assembly.uuid, resp['result'][0].uuid)
        self.assertEqual(fake_assembly.status, resp['result'][0].status)
        self.assertEqual(fake_assembly.user_id, resp['result'][0].user_id)
        self.assertEqual(fake_assembly.application_uri,
                         resp['result'][0].application_uri)
        hand_get.assert_called_with()
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp)

    @mock.patch('solum.objects.registry.Plan')
    def test_assemblies_post(self, mock_Plan, AssemblyHandler,
                             resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'plan_uri': 'http://test_url:8080/test/911'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        mock_Plan.get_by_uuid.return_value = fakes.FakePlan()

        hand_create = AssemblyHandler.return_value.create
        hand_create.return_value = fakes.FakeAssembly()
        assembly.AssembliesController().post()
        hand_create.assert_called_with({'name': 'foo',
                                        'plan_id': 8})
        mock_Plan.get_by_uuid.assert_called_with(None, '911')
        self.assertEqual(201, resp_mock.status)

    def test_assemblies_post_no_plan(self, AssemblyHandler, resp_mock,
                                     request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = AssemblyHandler.return_value.create
        hand_create.return_value = fakes.FakeAssembly()
        assembly.AssembliesController().post()
        ret_val = assembly.AssembliesController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertIn('The plan was not given or could not be found',
                      faultstring)
        self.assertEqual(400, resp_mock.status)

    def test_assemblies_post_not_hosted(self, AssemblyHandler, resp_mock,
                                        request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'plan_uri': 'http://example.com/a.git'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = AssemblyHandler.return_value.create
        hand_create.return_value = fakes.FakeAssembly()
        ret_val = assembly.AssembliesController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertIn('The plan was not hosted in solum', faultstring)
        self.assertEqual(400, resp_mock.status)

    def test_assem_post_nodata(self, AssemblyHandler, resp_mock,
                               request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        hand_create = AssemblyHandler.return_value.create
        hand_create.return_value = fakes.FakeAssembly()
        ret_val = assembly.AssembliesController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertEqual("Missing argument: \"data\"", faultstring)
        self.assertEqual(400, resp_mock.status)



# Copyright 2014 - Rackspace Hosting.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistral.actions import std_actions
from mistral import exceptions
from oslo_utils import uuidutils


class CreateImageAction(std_actions.HTTPAction):
    def __init__(self, build_service_url, action_context,
                 source_uri, name, base_image_id,
                 source_format, image_format):
        super(CreateImageAction, self).__init__(
            url='%s/v1/images' % build_service_url,
            method='POST',
            headers={'X-Auth-Token': action_context['openstack']['auth_token'],
                     'Content-Type': 'application/json',
                     'User-Agent': 'mistral-solum-plugin',
                     'Accept': 'application/json'},
            body={'name': name,
                  'source_uri': source_uri,
                  'source_format': source_format,
                  'base_image_id': base_image_id,
                  'image_format': image_format,
                  'type': 'image'},
            timeout=30)

    def run(self):
        return super(CreateImageAction, self).run()['content']

    def test(self):
        # This is just an example of what the builder returns.
        return {'created_image_id': '1-2-3-4',
                'state': 'PENDING',
                'uuid': '1-2-34'}

    def is_sync(self):
        return True


class GetImageIdAction(std_actions.HTTPAction):
    def __init__(self, build_service_url, action_context, uuid):
        super(GetImageIdAction, self).__init__(
            url='%s/v1/images/%s' % (build_service_url, uuid),
            method='GET',
            headers={'X-Auth-Token': action_context['openstack']['auth_token'],
                     'Content-Type': 'application/json',
                     'User-Agent': 'mistral-solum-plugin',
                     'Accept': 'application/json'})

    def run(self):
        resp = super(GetImageIdAction, self).run()
        created_image_id = resp['content'].get('created_image_id')
        if not uuidutils.is_uuid_like(created_image_id):
            raise exceptions.ActionException(
                'Image not built yet: image %(created_image_id)s,'
                ' state %(state)s' % resp['content'])
        return resp['content']

    def test(self):
        return {'created_image_id': '1-2-3-4',
                'state': 'PENDING',
                'uuid': '4-3-2-1'}

    def is_sync(self):
        return True



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum Worker shell handler."""

import ast
import base64
import json
import os
import random
import shelve
import string
import subprocess

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils
from sqlalchemy import exc as sqla_exc

import solum
from solum.common import clients
from solum.common import exception
from solum.common import repo_utils
from solum.conductor import api as conductor_api
from solum.deployer import api as deployer_api
from solum.i18n import _
from solum import objects
from solum.objects import assembly
from solum.objects import image
from solum.privileged import rootwrap as priv_rootwrap
import solum.uploaders.local as local_uploader
import solum.uploaders.swift as swift_uploader


LOG = logging.getLogger(__name__)

ASSEMBLY_STATES = assembly.States
IMAGE_STATES = image.States

cfg.CONF.import_opt('task_log_dir', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('proj_dir', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('param_file_path', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('log_upload_strategy', 'solum.worker.config',
                    group='worker')
cfg.CONF.import_opt('image_storage', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('temp_url_secret', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('temp_url_protocol', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('temp_url_ttl', 'solum.worker.config', group='worker')


def upload_task_log(ctxt, original_path, resource, build_id, stage):
    strategy = cfg.CONF.worker.log_upload_strategy
    LOG.debug("User log upload strategy: %s" % strategy)

    uploader = {
        'local': local_uploader.LocalStorage,
        'swift': swift_uploader.SwiftUpload,
    }.get(strategy, local_uploader.LocalStorage)
    uploader(ctxt, original_path, resource, build_id, stage).upload_log()


def job_update_notification(ctxt, build_id, status=None, description=None,
                            created_image_id=None, docker_image_name=None,
                            assembly_id=None):
    """send a status update to the conductor."""
    LOG.debug('build id:%s %s (%s) %s %s %s' % (build_id, status, description,
                                                created_image_id,
                                                docker_image_name,
                                                assembly_id),
              context=solum.TLS.trace)
    conductor_api.API(context=ctxt).build_job_update(build_id, status,
                                                     description,
                                                     created_image_id,
                                                     docker_image_name,
                                                     assembly_id)


def get_assembly_by_id(ctxt, assembly_id):
    return solum.objects.registry.Assembly.get_by_id(ctxt, assembly_id)


def get_image_by_id(ctxt, image_id):
    return solum.objects.registry.Image.get_by_id(ctxt, image_id)


def get_app_by_assem_id(ctxt, assembly_id):
    assem = get_assembly_by_id(ctxt, assembly_id)
    if assem:
        plan = solum.objects.registry.Plan.get_by_id(ctxt, assem.plan_id)
        app = solum.objects.registry.App.get_by_id(ctxt, plan.uuid)
        return app


def get_parameter_by_assem_id(ctxt, assembly_id):
    assem = get_assembly_by_id(ctxt, assembly_id)
    param_obj = solum.objects.registry.Parameter.get_by_plan_id(ctxt,
                                                                assem.plan_id)

    if not param_obj:
        plan = solum.objects.registry.Plan.get_by_id(ctxt, assem.plan_id)
        app = solum.objects.registry.App.get_by_id(ctxt, plan.uuid)
        app = json.loads(app.raw_content)
        param_obj = app.get('parameters', {})

    return param_obj


def update_assembly_status(ctxt, assembly_id, status):
    if assembly_id is None:
        return
    LOG.debug('Updating assembly %s status to %s' % (assembly_id, status))
    data = {'status': status}
    conductor_api.API(context=ctxt).update_assembly(assembly_id, data)
    try:
        update_wf_and_app_status(ctxt, assembly_id, status)
    except Exception as e:
        LOG.exception(e)


def update_wf_and_app_status(ctxt, assembly_id, status):
    # Update workflow and app objects
    status_data = dict()
    status_data['status'] = status
    try:
        wf = objects.registry.Workflow.get_by_assembly_id(assembly_id)
        objects.registry.Workflow.update_and_save(ctxt, wf.id, status_data)
    except sqla_exc.SQLAlchemyError as ex:
        LOG.error("Failed to update workflow corresponding to assembly %s"
                  % assembly_id)
        LOG.exception(ex)

    if wf is not None:
        try:
            app = objects.registry.App.get_by_id(ctxt, wf.app_id)
            objects.registry.App.update_and_save(ctxt, app.id, status_data)
        except sqla_exc.SQLAlchemyError as ex:
            LOG.error("Failed to update app status and app URL: %s" % app.id)
            LOG.exception(ex)


def update_lp_status(ctxt, image_id, name, status, external_ref=None,
                     docker_image_name=None):
    if image_id is None:
        return
    LOG.debug('Updating languagepack %s status to %s and external_ref to %s'
              % (name, status, external_ref))
    conductor_api.API(context=ctxt).update_image(image_id, status,
                                                 external_ref,
                                                 docker_image_name)


def get_lp_access_method(lp_project_id):
    if lp_project_id == cfg.CONF.api.operator_project_id:
        return 'operator'
    else:
        return 'custom'


class Handler(object):
    def echo(self, ctxt, message):
        LOG.debug("%s" % message)

    @exception.wrap_keystone_exception
    def get_du_details(self, ctxt, du_id):
        du_loc = None
        du_name = None
        du_image_backend = cfg.CONF.worker.image_storage

        if du_image_backend.lower() == 'glance':
            img = clients.OpenStackClients(ctxt).glance().images.get(du_id)
            du_loc = img.id
            du_name = img.name
        elif du_image_backend.lower() == 'swift':
            raise exception.NotImplemented()
        else:
            LOG.error("Invalid image storage option.")
            raise exception.ResourceNotFound()
        return du_loc, du_name

    @exception.wrap_keystone_exception
    def _get_environment(self, ctxt, git_info, assembly_id=None,
                         test_cmd=None, run_cmd=None, lp_access=None):
        source_uri = git_info['source_url']
        # create a minimal environment
        user_env = {}

        private = git_info.get('private', False)
        ssh_key = git_info.get('private_ssh_key', '')
        if private and ssh_key:
            user_env['REPO_DEPLOY_KEYS'] = ssh_key

        for var in ['PATH', 'LOGNAME', 'LANG', 'HOME', 'USER', 'TERM']:
            if var in os.environ:
                user_env[var] = os.environ[var]

        if assembly_id is not None:
            assem = get_assembly_by_id(ctxt, assembly_id)
            user_env['ASSEMBLY_ID'] = str(assem.uuid)
        else:
            str_assem = (''.join(random.choice(string.ascii_uppercase)
                         for i in range(20)))
            user_env['ASSEMBLY_ID'] = str_assem

        user_env['IMAGE_STORAGE'] = cfg.CONF.worker.image_storage
        user_env['DELETE_LOCAL_CACHE'] = cfg.CONF.worker.delete_local_cache

        if cfg.CONF.worker.image_storage == 'docker_registry':
            if cfg.CONF.worker.docker_reg_endpoint is None:
                LOG.debug("DU upload set to docker registry,")
                LOG.debug("but docker registry endpoint is not set.")
                LOG.debug("Setting it to 127.0.0.1")
                cfg.CONF.worker.docker_reg_endpoint = '127.0.0.1'
            user_env['DOCKER_REGISTRY'] = cfg.CONF.worker.docker_reg_endpoint
        else:
            client_region_name = clients.get_client_option('swift',
                                                           'region_name')
            user_env['OS_AUTH_TOKEN'] = ctxt.auth_token
            user_env['OS_AUTH_URL'] = ctxt.auth_url or ''
            user_env['OS_REGION_NAME'] = client_region_name
            kc = clients.OpenStackClients(ctxt).keystone()
            user_env['OS_IMAGE_URL'] = kc.client.service_catalog.url_for(
                service_type='image',
                interface='publicURL')
            user_env['OS_STORAGE_URL'] = kc.client.service_catalog.url_for(
                service_type='object-store',
                interface='publicURL',
                region_name=client_region_name)
            user_env['TEMP_URL_SECRET'] = cfg.CONF.worker.temp_url_secret
            user_env['TEMP_URL_PROTOCOL'] = cfg.CONF.worker.temp_url_protocol
            user_env['TEMP_URL_TTL'] = cfg.CONF.worker.temp_url_ttl
            user_env['OPR_LP_DOWNLOAD_STRATEGY'] = (
                cfg.CONF.worker.operator_lp_download_strategy)

            # Get LP Operator context for downloading operator LPs
            lp_kc = clients.OpenStackClients(None).keystone().lp_admin_client
            # Get the auth ref from session
            auth_ref = lp_kc.session.auth.get_auth_ref(lp_kc.session)
            # Get service_catalog
            service_catalog = auth_ref.service_catalog
            user_env['OPER_AUTH_TOKEN'] = lp_kc.session.get_token()
            user_env['OPER_OS_STORAGE_URL'] = service_catalog.url_for(
                service_type='object-store',
                interface='publicURL',
                region_name=client_region_name)

        if test_cmd is not None:
            user_env['TEST_CMD'] = test_cmd
        if run_cmd is not None:
            user_env['RUN_CMD'] = run_cmd

        user_env['PROJECT_ID'] = ctxt.project_id

        user_env['BUILD_ID'] = uuidutils.generate_uuid()
        user_env['SOLUM_TASK_DIR'] = cfg.CONF.worker.task_log_dir

        if lp_access is not None:
            user_env['ACCESS'] = lp_access

        params_env = self._get_parameter_env(ctxt, source_uri, assembly_id,
                                             user_env['BUILD_ID'])
        user_env.update(params_env)
        return user_env

    @property
    def proj_dir(self):
        if cfg.CONF.worker.proj_dir:
            return cfg.CONF.worker.proj_dir
        return os.path.abspath(os.path.join(os.path.dirname(__file__),
                                            '..', '..', '..'))

    def _get_build_command(self, ctxt, stage, source_uri, name,
                           base_image_id, source_format, image_format,
                           commit_sha, artifact_type=None, lp_image_tag=None):

        # map the input formats to script paths.
        # TODO(asalkeld) we need an "auto".
        pathm = {'heroku': 'lp-cedarish',
                 'dib': 'diskimage-builder',
                 'dockerfile': 'lp-dockerfile',
                 'chef': 'lp-chef',
                 'docker': 'docker',
                 'qcow2': 'docker',
                 'vm': 'docker'}
        if base_image_id == 'auto' and image_format == 'qcow2':
            base_image_id = 'cedarish'
        build_app_path = os.path.join(self.proj_dir, 'contrib',
                                      pathm.get(source_format, 'lp-cedarish'),
                                      pathm.get(image_format, 'docker'))

        if artifact_type == 'language_pack':
            build_lp = os.path.join(build_app_path, 'build-lp')
            return [build_lp, source_uri, name, ctxt.project_id]

        if stage == 'unittest':
            build_app = os.path.join(build_app_path, 'unittest-app')
            return [build_app, source_uri, commit_sha, ctxt.project_id,
                    base_image_id, lp_image_tag]
        elif stage == 'build':
            build_app = os.path.join(build_app_path, 'build-app')
            return [build_app, source_uri, commit_sha, name, ctxt.project_id,
                    base_image_id, lp_image_tag]

    def _get_parameter_env(self, ctxt, source_uri, assembly_id, build_id):
        param_env = {}
        if assembly_id is None:
            return param_env

        param_obj = get_parameter_by_assem_id(ctxt, assembly_id)
        if param_obj is None:
            return param_env

        user_param_file = '/'.join([cfg.CONF.worker.param_file_path,
                                    build_id, 'user_params'])
        solum_param_file = '/'.join([cfg.CONF.worker.param_file_path,
                                     build_id, 'solum_params'])
        try:
            os.makedirs(os.path.dirname(user_param_file), 0o700)
        except OSError as ex:
            LOG.error("Error creating dirs to write out param files, %s" % ex)
            return param_env

        def _sanitize_param(s):
            if s is None:
                return ''
            elif isinstance(s, str):
                # Handles the case of exporting a var with a multi-line string
                return ''.join(['"', s.strip('\n').replace('"', '\\"'), '"'])
            else:
                return str(s)

        with open(user_param_file, 'w') as f:
            f.write("#!/bin/bash\n")
            if param_obj.get('user_params'):
                for k, v in param_obj['user_params'].items():
                    if k and k.startswith('_SYSTEM'):
                        # Variables for control purpose, e.g. _SYSTEM_USE_DRONE
                        param_env[k] = _sanitize_param(v)
                    else:
                        f.write("export %s=%s\n" % (k, _sanitize_param(v)))
        with open(solum_param_file, 'w') as f:
            f.write("#!/bin/bash\n")
            if param_obj.get('solum_params'):
                for k, v in param_obj['solum_params'].items():
                    if k == 'REPO_DEPLOY_KEYS':
                        # Pass in deploy key as an environment variable
                        param_env[k] = self._get_private_key(v, source_uri)
                    f.write("export %s=%s\n" % (k, _sanitize_param(v)))

        param_env['USER_PARAMS'] = user_param_file
        param_env['SOLUM_PARAMS'] = solum_param_file
        return param_env

    def launch_workflow(self, ctxt, build_id, git_info, ports, name,
                        base_image_id, source_format, image_format,
                        assembly_id, workflow, test_cmd, run_cmd, du_id):

        if 'unittest' in workflow:
            if self._do_unittest(ctxt, build_id, git_info, name, base_image_id,
                                 source_format, image_format, assembly_id,
                                 test_cmd) != 0:
                return

        du_image_loc = None
        du_image_name = None
        if 'build' in workflow:
            du_image_loc, du_image_name = self._do_build(
                ctxt, build_id, git_info, name, base_image_id, source_format,
                image_format, assembly_id, run_cmd)

        if 'deploy' in workflow:
            if du_id:
                du_image_loc, du_image_name = self.get_du_details(ctxt, du_id)
            if du_image_loc and du_image_name:
                self._do_deploy(ctxt, assembly_id, ports, du_image_loc,
                                du_image_name)
            else:
                LOG.warning("Deploy called without DU details. "
                            "Cannot continue.")
                return

        if 'scale' in workflow:
            self._do_scale(ctxt, assembly_id)

    def build(self, ctxt, build_id, git_info, name, base_image_id,
              source_format, image_format, assembly_id, run_cmd):
        self._do_build(ctxt, build_id, git_info, name, base_image_id,
                       source_format, image_format, assembly_id, run_cmd)

    def unittest(self, ctxt, build_id, git_info, name, base_image_id,
                 source_format, image_format, assembly_id, test_cmd):
        self._do_unittest(ctxt, build_id, git_info, name, base_image_id,
                          source_format, image_format, assembly_id, test_cmd)

    def _do_deploy(self, ctxt, assembly_id, ports, du_image_loc,
                   du_image_name):
        app = get_app_by_assem_id(ctxt, assembly_id)
        LOG.debug("Deploying app %s %s" % (app.name, app.id))
        deployer_api.API(context=ctxt).deploy(assembly_id=assembly_id,
                                              image_loc=du_image_loc,
                                              image_name=du_image_name,
                                              ports=ports)

    def _do_scale(self, ctxt, assembly_id):
        app = get_app_by_assem_id(ctxt, assembly_id)
        LOG.debug("Scaling app %s %s" % (app.name, app.id))
        deployer_api.API(context=ctxt).scale(assembly_id=assembly_id)

    def _do_build(self, ctxt, build_id, git_info, name, base_image_id,
                  source_format, image_format, assembly_id, run_cmd):
        update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.BUILDING)

        app = get_app_by_assem_id(ctxt, assembly_id)
        LOG.debug("Building app %s %s" % (app.name, app.id))

        solum.TLS.trace.clear()
        solum.TLS.trace.import_context(ctxt)

        source_uri = git_info['source_url']
        commit_sha = git_info.get('commit_sha', '')
        private = git_info.get('private', False)
        ssh_key = git_info.get('private_ssh_key', '')
        # If the repo is private, make sure private ssh key is provided
        if private and not ssh_key:
            LOG.warning("Error building due to missing private ssh key."
                        " assembly ID: %s" % assembly_id)
            job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                    description='private ssh key missing',
                                    assembly_id=assembly_id)
            update_assembly_status(ctxt, assembly_id,
                                   ASSEMBLY_STATES.ERROR)
            return

        image_tag = ''
        lp_access = ''
        if base_image_id != 'auto':
            image = objects.registry.Image.get_lp_by_name_or_uuid(
                ctxt, base_image_id, include_operators_lp=True)
            if (not image or not image.project_id or not image.status or
                    not image.external_ref or not image.docker_image_name or
                    image.status.lower() != 'ready'):
                LOG.warning("Error building due to language pack not ready."
                            " assembly ID: %s" % assembly_id)
                job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                        description='language pack not ready',
                                        assembly_id=assembly_id)
                update_assembly_status(ctxt, assembly_id,
                                       ASSEMBLY_STATES.ERROR)
                return
            base_image_id = image.external_ref
            image_tag = image.docker_image_name
            lp_access = get_lp_access_method(image.project_id)

        build_cmd = self._get_build_command(ctxt, 'build', source_uri,
                                            name, base_image_id,
                                            source_format, image_format,
                                            commit_sha,
                                            lp_image_tag=image_tag)
        solum.TLS.trace.support_info(build_cmd=' '.join(build_cmd),
                                     assembly_id=assembly_id)

        user_env = {}
        try:
            user_env = self._get_environment(ctxt,
                                             git_info,
                                             assembly_id=assembly_id,
                                             run_cmd=run_cmd,
                                             lp_access=lp_access)
        except exception.SolumException as env_ex:
            LOG.exception(env_ex)
            job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                    description=str(env_ex),
                                    assembly_id=assembly_id)

        log_env = user_env.copy()
        if 'OS_AUTH_TOKEN' in log_env:
            del log_env['OS_AUTH_TOKEN']
        if 'OPER_AUTH_TOKEN' in log_env:
            del log_env['OPER_AUTH_TOKEN']
        if 'OPER_OS_STORAGE_URL' in log_env:
            del log_env['OPER_OS_STORAGE_URL']
        solum.TLS.trace.support_info(environment=log_env)

        job_update_notification(ctxt, build_id, IMAGE_STATES.BUILDING,
                                description='Starting the image build',
                                assembly_id=assembly_id)
        # TODO(datsun180b): Associate log with assembly properly
        logpath = "%s/%s-%s.log" % (user_env['SOLUM_TASK_DIR'],
                                    'build',
                                    user_env['BUILD_ID'])
        LOG.debug("Build logs for app %s stored at %s" % (app.name, logpath))
        out = None
        assem = None
        if assembly_id is not None:
            assem = get_assembly_by_id(ctxt, assembly_id)
            if assem.status == ASSEMBLY_STATES.DELETING:
                return

        try:
            out = subprocess.Popen(build_cmd,
                                   env=user_env,
                                   stdout=subprocess.PIPE).communicate()[0]
        except (OSError, ValueError) as subex:
            LOG.exception(subex)
            job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                    description=str(subex),
                                    assembly_id=assembly_id)
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.ERROR)
            return

        if assem is not None:
            assem.type = 'app'
            wf = objects.registry.Workflow.get_by_assembly_id(assem.id)
            upload_task_log(ctxt, logpath, assem, wf.id, 'build')

        '''
        we expect two lines in the output that looks like:
        created_image_id=<location of DU>
        docker_image_name=<DU name>
        The DU location is:
        DU's swift tempUrl if backend is 'swift';
        DU's UUID in glance if backend is 'glance';
        DU's docker registry location if backend is 'docker_registry'
        '''
        du_image_loc = None
        docker_image_name = None
        for line in out.split('\n'):
            # Won't break out until we get the final
            # matching which is the expected value
            if line.startswith('created_image_id'):
                solum.TLS.trace.support_info(build_out_line=line)
                du_image_loc = line.replace('created_image_id=', '').strip()
            elif line.startswith('docker_image_name'):
                docker_image_name = line.replace('docker_image_name=', '')

        if not du_image_loc or not docker_image_name:
            job_update_notification(ctxt, build_id, IMAGE_STATES.ERROR,
                                    description='image not created',
                                    assembly_id=assembly_id)
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.ERROR)
            return
        else:
            job_update_notification(ctxt, build_id, IMAGE_STATES.READY,
                                    description='built successfully',
                                    created_image_id=du_image_loc,
                                    docker_image_name=docker_image_name,
                                    assembly_id=assembly_id)
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.BUILT)
            return (du_image_loc, docker_image_name)

    def _do_unittest(self, ctxt, build_id, git_info, name, base_image_id,
                     source_format, image_format, assembly_id, test_cmd):
        if test_cmd is None:
            LOG.debug("Unit test command is None; skipping unittests.")
            return 0

        app = get_app_by_assem_id(ctxt, assembly_id)
        LOG.debug("Unit testing for app %s %s" % (app.name, app.id))

        commit_sha = git_info.get('commit_sha', '')
        status_url = git_info.get('status_url')
        repo_token = git_info.get('repo_token')

        update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.UNIT_TESTING)

        image_tag = ''
        lp_access = ''
        if base_image_id != 'auto':
            image = objects.registry.Image.get_lp_by_name_or_uuid(
                ctxt, base_image_id, include_operators_lp=True)
            if (not image or not image.project_id or not image.status or
                    not image.external_ref or not image.docker_image_name or
                    image.status.lower() != 'ready'):
                LOG.warning("Error running unittest due to language pack"
                            " not ready. assembly ID: %s" % assembly_id)
                update_assembly_status(ctxt, assembly_id,
                                       ASSEMBLY_STATES.ERROR)
                return
            base_image_id = image.external_ref
            image_tag = image.docker_image_name
            lp_access = get_lp_access_method(image.project_id)

        git_url = git_info['source_url']
        command = self._get_build_command(ctxt, 'unittest', git_url, name,
                                          base_image_id,
                                          source_format, image_format,
                                          commit_sha, lp_image_tag=image_tag)

        solum.TLS.trace.clear()
        solum.TLS.trace.import_context(ctxt)

        user_env = self._get_environment(ctxt,
                                         git_info,
                                         assembly_id=assembly_id,
                                         test_cmd=test_cmd,
                                         lp_access=lp_access)
        log_env = user_env.copy()
        if 'OS_AUTH_TOKEN' in log_env:
            del log_env['OS_AUTH_TOKEN']
        if 'OPER_AUTH_TOKEN' in log_env:
            del log_env['OPER_AUTH_TOKEN']
        if 'OPER_OS_STORAGE_URL' in log_env:
            del log_env['OPER_OS_STORAGE_URL']
        solum.TLS.trace.support_info(environment=log_env)

        logpath = "%s/%s-%s.log" % (user_env['SOLUM_TASK_DIR'],
                                    'unittest',
                                    user_env['BUILD_ID'])
        LOG.debug("Unittest logs stored at %s" % logpath)

        returncode = -1
        assem = None
        if assembly_id is not None:
            assem = get_assembly_by_id(ctxt, assembly_id)
            if assem.status == ASSEMBLY_STATES.DELETING:
                return returncode

        try:
            runtest = subprocess.Popen(command, env=user_env,
                                       stdout=subprocess.PIPE)
            returncode = runtest.wait()
        except OSError as subex:
            LOG.exception("Exception running unit tests:")
            LOG.exception(subex)

        if assem is not None:
            assem.type = 'app'
            wf = objects.registry.Workflow.get_by_assembly_id(assem.id)
            upload_task_log(ctxt, logpath, assem, wf.id, 'unittest')

        if returncode == 0:
            update_assembly_status(ctxt, assembly_id,
                                   ASSEMBLY_STATES.UNIT_TESTING_PASSED)
        elif returncode > 0:
            LOG.error("Unit tests failed. Return code is %r" % (returncode))
            update_assembly_status(ctxt, assembly_id,
                                   ASSEMBLY_STATES.UNIT_TESTING_FAILED)
        elif returncode < 0:
            LOG.error("Error running unit tests.")
            update_assembly_status(ctxt, assembly_id, ASSEMBLY_STATES.ERROR)

        repo_utils.send_status(returncode, status_url, repo_token)

        return returncode

    def _get_private_key(self, source_creds, source_url):
        source_private_key = ''
        if source_creds:
            cfg.CONF.import_opt('system_param_store',
                                'solum.api.handlers.plan_handler',
                                group='api')
            store = cfg.CONF.api.system_param_store

            if store == 'database':
                deploy_keys_str = base64.b64decode(source_creds)
            elif store == 'barbican':
                client = clients.OpenStackClients(None).barbican().admin_client
                secret = client.secrets.get(secret_ref=source_creds)
                deploy_keys_str = secret.payload
            elif store == 'local_file':
                cfg.CONF.import_opt('system_param_file',
                                    'solum.api.handlers.plan_handler',
                                    group='api')
                secrets_file = cfg.CONF.api.system_param_file
                s = shelve.open(secrets_file)
                deploy_keys_str = s[str(source_creds)]
                deploy_keys_str = base64.b64decode(deploy_keys_str)
                s.close()
            deploy_keys = ast.literal_eval(deploy_keys_str)
            for dk in deploy_keys:
                if source_url == dk['source_url']:
                    source_private_key = dk['private_key']
        return source_private_key

    def build_lp(self, ctxt, image_id, git_info, name, source_format,
                 image_format, artifact_type, lp_params):

        LOG.debug("Building languagepack %s" % name)
        update_lp_status(ctxt, image_id, name, IMAGE_STATES.BUILDING)

        solum.TLS.trace.clear()
        solum.TLS.trace.import_context(ctxt)

        source_uri = git_info['source_url']
        build_cmd = self._get_build_command(ctxt, 'build', source_uri,
                                            name, str(image_id),
                                            source_format, 'docker', '',
                                            artifact_type)

        lp_access = get_lp_access_method(ctxt.project_id)

        user_env = {}
        try:
            user_env = self._get_environment(ctxt,
                                             git_info,
                                             lp_access=lp_access)
        except exception.SolumException as env_ex:
            LOG.exception(_("Failed to successfully get environment for "
                            "building languagepack: `%s`"),
                          image_id)
            LOG.exception(env_ex)

        log_env = user_env.copy()
        if 'OS_AUTH_TOKEN' in log_env:
            del log_env['OS_AUTH_TOKEN']
        if 'OPER_AUTH_TOKEN' in log_env:
            del log_env['OPER_AUTH_TOKEN']
        if 'OPER_OS_STORAGE_URL' in log_env:
            del log_env['OPER_OS_STORAGE_URL']
        solum.TLS.trace.support_info(environment=log_env)

        logpath = "%s/%s-%s.log" % (user_env['SOLUM_TASK_DIR'],
                                    'languagepack',
                                    user_env['BUILD_ID'])
        LOG.debug("Languagepack logs for LP %s stored at %s" %
                  (image_id, logpath))

        out = None
        status = IMAGE_STATES.ERROR
        image_external_ref = None
        docker_image_name = None

        try:
            try:
                out = priv_rootwrap.execute(
                    *build_cmd, run_as_root=True, env_variables=user_env)[0]
            except Exception as e:
                LOG.exception("Failed to build languagepack: %s" % image_id)
                LOG.exception(e)
                out = ''

            if isinstance(out, bytes):
                out = out.decode('utf-8')
            # we expect two lines in the output that looks like:
            # image_external_ref=<external storage ref>
            # docker_image_name=<DU name>
            for line in out.split('\n'):
                # Won't break out until we get the final
                # matching which is the expected value
                if line.startswith('image_external_ref'):
                    solum.TLS.trace.support_info(build_lp_out_line=line)
                    image_external_ref = line.replace('image_external_ref=',
                                                      '').strip()
                elif line.startswith('docker_image_name'):
                    docker_image_name = line.replace('docker_image_name=', '')
            if image_external_ref and docker_image_name:
                status = IMAGE_STATES.READY
            else:
                status = IMAGE_STATES.ERROR
        except OSError as subex:
            LOG.exception(_("Failed to successfully build languagepack: `%s`"),
                          image_id)
            LOG.exception(subex)

        img = get_image_by_id(ctxt, image_id)
        img.type = 'languagepack'
        update_lp_status(ctxt, image_id, name, status, image_external_ref,
                         docker_image_name)
        upload_task_log(ctxt, logpath, img,
                        img.uuid, 'languagepack')



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import requests
import socket

import httplib2
from oslo_config import cfg
from oslo_log import log as logging

from solum.common import exception


LOG = logging.getLogger(__name__)

http_timeout_opt = [
    cfg.IntOpt('http_request_timeout',
               default=2,
               help='Timeout value in seconds for sending http requests')
]


def list_opts():
    yield 'api', http_timeout_opt


CONF = cfg.CONF
CONF.register_opts(http_timeout_opt, group='api')

CONF.import_opt('log_url_prefix', 'solum.worker.config', group='worker')
http_req_timeout = CONF.api.http_request_timeout


def get_http_connection():
    return httplib2.Http(timeout=http_req_timeout)


def is_reachable(url):
    reachable = False
    conn = get_http_connection()
    resp, _ = conn.request(url, 'GET')
    if resp is not None and resp['status'] == '200':
        reachable = True
    return reachable


def send_status(test_result, status_url, repo_token, pending=False):
    if status_url and repo_token:
        commit_id = status_url.rstrip('/').split('/')[-1]
        log_url = cfg.CONF.worker.log_url_prefix + commit_id
        headers = {'Authorization': 'token ' + repo_token,
                   'Content-Type': 'application/json'}
        if pending:
            data = {'state': 'pending',
                    'description': 'Solum says: Testing in progress',
                    'target_url': log_url}
        elif test_result == 0:
            data = {'state': 'success',
                    'description': 'Solum says: Tests passed',
                    'target_url': log_url}
        else:
            data = {'state': 'failure',
                    'description': 'Solum says: Tests failed',
                    'target_url': log_url}

        resp = requests.post(status_url, headers=headers,
                             data=json.dumps(data))
        if resp.status_code != '201':
            LOG.debug("Failed to send back status. Error code %s,"
                      "status_url %s, repo_token %s" %
                      (resp.status_code, status_url, repo_token))
    else:
        LOG.debug("No url or token available to send back status")


def verify_artifact(artifact, collab_url):
    # TODO(james_li): verify if the artifact is the one that gets triggered
    if collab_url is None:
        return True

    repo_token = artifact.get('repo_token')
    if repo_token is None:
        return False

    headers = {'Authorization': 'token ' + repo_token}
    try:
        conn = get_http_connection()
        resp, _ = conn.request(collab_url, headers=headers)
        if resp['status'] == '204':
            return True
        elif resp['status'] == '404':
            raise exception.RequestForbidden(
                reason="User %s not allowed to do rebuild" %
                       collab_url.split('/')[-1])
    except (httplib2.HttpLib2Error, socket.error) as ex:
        LOG.warning("Error in verifying collaborator, collab url: %s,"
                    " error: %s" % (collab_url, ex))
    return False



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

import pecan
from pecan import core

from solum.api.controllers.camp.v1_1.datamodel import types as camp_types
from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types
from solum.api.handlers.camp import attribute_definition_handler


class AttributeLink(common_types.Link):
    """CAMP v1.1 AttributeLink

    Links type_definition resources to attribute_definition resources.
    """

    required = camp_types.BooleanType
    """The value of the required attribute determines if the attribute defined
    by the attribute_definition resource referenced by this AttributeLink is
    required for resources of the type defined by the containing
    type_definition resource.
    """

    mutable = camp_types.BooleanType
    """The value of the mutable attribute determines if the attribute defined
    by the attribute_definition resource referenced by this AttributeLink is
    mutable for resources of the type defined by the containing
    type_definition resource.
    """

    consumer_mutable = camp_types.BooleanType
    """The value of the consumer_mutable attribute determines if the attribute
    defined by the attribute_definition resource referenced by this
    AttributeLink is writable by Consumers for resources of the type defined
    by the containing type_definition resource.
    """

    @classmethod
    def from_json(cls, dct):
        ret_val = cls()
        for key, value in dct.items():
            if hasattr(ret_val, key):
                setattr(ret_val, key, value)
        return ret_val

    def fix_uris(self, host_url):
        handler = (attribute_definition_handler.
                   AttributeDefinitionHandler(pecan.request.security_context))
        raw_def = handler.get(self.href)
        if not raw_def:
            core.abort(404,
                       'no attribute definition_link for %s' %
                       self.href)
        self.target_name = raw_def.name
        self.href = uris.ATTRIBUTE_DEF_URI_STR % (host_url, self.href)


class TypeDefinition(api_types.Base):
    """CAMP v1.1 type_definition resource."""

    documentation = common_types.Uri
    """This attribute contains a URI that points to the documentation for the
     resource type; required. not mutable."""

    inherits_from = [common_types.Link]
    """Each Link in this array points to a type_definition resource that the
     described resource's type inherits from; not required, not mutable."""

    attribute_definition_links = [AttributeLink]
    """Each AttributeLink in this array references an attribute_definition
     resource. Each of these attribute_definition resources represents an
     attribute of the type described by this type_definition resource;
     required, not mutable."""

    def __init__(self, **kwds):
        super(TypeDefinition, self).__init__(**kwds)

    @classmethod
    def from_json(cls, dct):
        ret_val = cls()
        for key, value in dct.items():
            if key == 'inherits_from':
                inherit_links = []
                for l_dict in value:
                    link = common_types.Link(href=l_dict['href'],
                                             target_name=l_dict['target_name'])
                    inherit_links.append(link)
                setattr(ret_val, 'inherits_from', inherit_links)
            elif key == 'attribute_definition_links':
                ad_links = []
                for ad_dct in value:
                    ad_link = AttributeLink.from_json(ad_dct)
                    ad_links.append(ad_link)
                setattr(ret_val, 'attribute_definition_links', ad_links)
            elif hasattr(ret_val, key):
                setattr(ret_val, key, value)
            else:
                core.abort(500, 'internal metadata is incorrect')
        return ret_val

    def fix_uris(self, host_url):
        """Update URIs to reflect a host URL."""
        ret_val = copy.deepcopy(self)
        ret_val.uri = uris.TYPE_DEF_URI_STR % (host_url, ret_val.uri)

        # 'inherits_from' is optional
        if ret_val.inherits_from:
            for ih_link in ret_val.inherits_from:
                ih_link.href = uris.TYPE_DEF_URI_STR % (host_url, ih_link.href)

        # 'attribute_definition_links' is required
        for ad_link in ret_val.attribute_definition_links:
            ad_link.fix_uris(host_url)

        return ret_val


class TypeDefinitions(api_types.Base):
    """CAMP v1.1 type_definitions resource."""

    type_definition_links = [common_types.Link]
    """This attribute contains Links to type_definition resources that contain
    information about resource types supported by the Platform.
    """

    def __init__(self, **kwds):
        super(TypeDefinitions, self).__init__(**kwds)



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import language_pack
import solum.api.controllers.v1.userlog as userlog_controller
from solum.api.handlers import language_pack_handler
from solum.common import exception
from solum.common import policy
from solum import objects


class LanguagePackController(rest.RestController):
    """Manages operations on a single languagepack."""

    def __init__(self, lp_id):
        super(LanguagePackController, self).__init__()
        self._id = lp_id

    @pecan.expose()
    def _lookup(self, primary_key, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        if primary_key == 'logs':
            logs = userlog_controller.UserlogsController(self._id)
            return logs, remainder

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(language_pack.LanguagePack)
    def get(self):
        """Return a languagepack."""
        policy.check('show_languagepack',
                     pecan.request.security_context)
        handler = language_pack_handler.LanguagePackHandler(
            pecan.request.security_context)

        host_url = pecan.request.application_url.rstrip('/')
        return language_pack.LanguagePack.from_db_model(handler.get(self._id),
                                                        host_url)

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=204)
    def delete(self):
        """Delete a languagepack."""
        policy.check('delete_languagepack',
                     pecan.request.security_context)
        handler = language_pack_handler.LanguagePackHandler(
            pecan.request.security_context)
        return handler.delete(self._id)


class LanguagePacksController(rest.RestController):
    """Manages operations on the languagepack collection."""

    @pecan.expose()
    def _lookup(self, lp_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return LanguagePackController(lp_id), remainder

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(language_pack.LanguagePack,
                         body=language_pack.LanguagePack,
                         status_code=201)
    def post(self, data):
        """Create a new languagepack."""
        policy.check('create_languagepack',
                     pecan.request.security_context)
        handler = language_pack_handler.LanguagePackHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')

        msg = ("Languagepack name must be 1-100 characters long, only contain "
               "a-z,0-9,-,_ and start with an alphabet character.")
        if not data.name or not data.name[0].isalpha():
            raise exception.BadRequest(reason=msg)

        try:
            re.match(r'^([a-z0-9-_]{1,100})$', data.name).group(0)
        except AttributeError:
            raise exception.BadRequest(reason=msg)

        return language_pack.LanguagePack.from_db_model(
            handler.create(data.as_dict(objects.registry.Image),
                           data.lp_metadata, data.lp_params), host_url)

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose([language_pack.LanguagePack])
    def get_all(self):
        """Return all languagepacks, based on the query provided."""
        policy.check('get_languagepacks',
                     pecan.request.security_context)
        handler = language_pack_handler.LanguagePackHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [language_pack.LanguagePack.from_db_model(img, host_url)
                for img in handler.get_all()]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.controllers.camp.v1_1 import attribute_definitions
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.camp.attribute_definition_handler.'
            'AttributeDefinitionHandler')
class TestAttributeDefinitions(base.BaseTestCase):
    def setUp(self):
        super(TestAttributeDefinitions, self).setUp()
        objects.load()

    def test_attribute_definition_get(self,
                                      AttributeDefinitionHandler,
                                      resp_mock,
                                      request_mock):
        hand_get = AttributeDefinitionHandler.return_value.get
        fake_attribute_definition = fakes.FakeAttributeDefinition()
        hand_get.return_value = fake_attribute_definition

        cont = attribute_definitions.AttributeDefinitionsController()
        resp = cont.get_one('some_attribute')
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertEqual(fake_attribute_definition.type, resp['result'].type)
        self.assertEqual(fake_attribute_definition.name, resp['result'].name)

    def test_attribute_def_get_not_found(self,
                                         AttributeDefinitionHandler,
                                         resp_mock,
                                         request_mock):
        hand_get = AttributeDefinitionHandler.return_value.get
        hand_get.return_value = None

        cont = attribute_definitions.AttributeDefinitionsController()
        resp = cont.get_one('some_attribute')
        self.assertIsNotNone(resp)
        self.assertEqual(404, resp_mock.status)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from io import StringIO
from requests import exceptions
from unittest import mock
import urllib

from solum.common import urlfetch
from solum.tests import base


FETCH_SIZE_OK = 500


class Response(object):
    def __init__(self, buf=''):
        self.buf = buf

    def iter_content(self, chunk_size=1):
        while self.buf:
            yield self.buf[:chunk_size]
            self.buf = self.buf[chunk_size:]

    def raise_for_status(self):
        pass


class TestUrlFetch(base.BaseTestCase):
    def setUp(self):
        super(TestUrlFetch, self).setUp()

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_max_size_zero_byte(self, mock_get):
        data = '{ "foo": "bar" }'
        mock_get.return_value = Response(data)
        url = 'http://example.com/plan'
        self.assertRaises(IOError, urlfetch.get, url, 0)

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_chunk_size_zero_byte(self, mock_get):
        data = '{ "foo": "bar" }'
        mock_get.return_value = Response(data)
        url = 'http://example.com/plan'
        self.assertRaises(IOError, urlfetch.get, url, FETCH_SIZE_OK, 0)

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_http_scheme(self, mock_get):
        data = '{ "foo": "bar" }'
        mock_get.return_value = Response(data)
        url = 'http://example.com/plan'
        self.assertEqual(data, urlfetch.get(url, FETCH_SIZE_OK))

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_https_scheme(self, mock_get):
        url = 'https://example.com/plan'
        data = '{ "foo": "bar" }'
        mock_get.return_value = Response(data)
        self.assertEqual(data, urlfetch.get(url, FETCH_SIZE_OK))

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_http_error(self, mock_get):
        url = 'http://example.com/plan'
        mock_get.side_effect = IOError
        self.assertRaises(IOError, urlfetch.get, url, FETCH_SIZE_OK)

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_non_exist_url(self, mock_get):
        url = 'http://non-exist.com/plan'
        mock_get.side_effect = exceptions.Timeout
        self.assertRaises(IOError, urlfetch.get, url, FETCH_SIZE_OK)

    def test_invalid_url(self):
        self.assertRaises(IOError, urlfetch.get, 'invalid_url', FETCH_SIZE_OK)

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_max_fetch_size_okay(self, mock_get):
        url = 'http://example.com/plan'
        data = '{ "foo": "bar" }'
        mock_get.return_value = Response(data)
        urlfetch.get(url, FETCH_SIZE_OK)

    @mock.patch('solum.common.urlfetch.requests.get')
    def test_max_fetch_size_error(self, mock_get):
        url = 'http://example.com/plan'
        data = '{ "foo": "bar" }'
        mock_get.return_value = Response(data)
        exception = self.assertRaises(IOError, urlfetch.get, url, 5)
        self.assertIn("File exceeds", str(exception))

    def test_file_scheme_default_behaviour(self):
        self.assertRaises(IOError, urlfetch.get, 'file:///etc/profile',
                          FETCH_SIZE_OK)

    @mock.patch('urllib.request.urlopen')
    def test_file_scheme_supported(self, mock_urlopen):
        data = '{ "foo": "bar" }'
        url = 'file:///etc/profile'
        mock_urlopen.return_value = StringIO(data)
        self.assertEqual(data, urlfetch.get(url, FETCH_SIZE_OK,
                                            allowed_schemes=['file']))

    @mock.patch('urllib.request.urlopen')
    def test_file_scheme_failure(self, mock_urlopen):
        url = 'file:///etc/profile'
        mock_urlopen.side_effect = urllib.error.URLError('oops')
        self.assertRaises(IOError, urlfetch.get, url, FETCH_SIZE_OK,
                          allowed_schemes=['file'])



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""extend base_image_id to 100 chars

Revision ID: 43df309dbf3f
Revises: 40df542e345
Create Date: 2015-06-30 20:39:56.988397

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '43df309dbf3f'
down_revision = '40df542e345'


def upgrade():
    op.alter_column('image', 'base_image_id',
                    type_=sa.String(length=100),
                    existing_type=sa.String(length=36),
                    existing_nullable=True,
                    )


def downgrade():
    op.alter_column('image', 'base_image_id',
                    type_=sa.String(length=36),
                    existing_type=sa.String(length=100),
                    existing_nullable=True,
                    )



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""empty message

Revision ID: a271b35952be
Revises: 272e7cd352f9
Create Date: 2016-02-03 15:46:27.641527

"""
from alembic import op
import sqlalchemy as sa

from solum.objects.sqlalchemy import models

# revision identifiers, used by Alembic.
revision = 'a271b35952be'
down_revision = '272e7cd352f9'


def upgrade():
    op.add_column('app',
                  sa.Column('scale_config', models.JSONEncodedDict(1024)))


def downgrade():
    op.drop_column('app', 'scale_config')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json
from urllib import parse

from oslo_config import cfg
from oslo_log import log as logging
import pecan
from pecan import rest

from solum.api.handlers import app_handler
from solum.common import exception
from solum.common import policy

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


def query_dict(querystring):
    if not querystring:
        return {}
    query = parse.unquote(querystring).rstrip()
    query = query.split('&')
    query = [q.split('=') for q in query]
    return dict([(q[0], ' '.join(q[1:])) for q in query])


class TriggerController(rest.RestController):
    """Manages triggers."""

    def _process_request(self, body):
        commit_sha = ''
        collab_url = None

        phrase = body['comment']['body']
        commenter = body['comment']['user']['login']
        private_repo = body['repository']['private']
        # An example of collab_url
        # https://api.github.com/repos/:user/:repo/collaborators{/collaborator}
        if not private_repo:
            # Only verify collaborator for public repos
            collab_url = (
                body['repository']['collaborators_url'].format(
                    **{'/collaborator': '/' + commenter}))
        if (phrase.strip('. ').lower() !=
                CONF.api.rebuild_phrase.lower()):
            err_msg = 'Rebuild phrase does not match'
            raise exception.RequestForbidden(reason=err_msg)
        else:
            commit_sha = body['comment']['commit_id']
        return commit_sha, collab_url

    def _get_workflow(self, query):
        workflow = None

        if 'workflow' in query:
            valid_stages = ['unittest', 'build', 'deploy']
            workflow = query['workflow'].replace('+', ' ').split(' ')
            workflow = filter(lambda x: x in valid_stages, workflow)
            if not workflow:
                workflow = None
        return workflow

    @exception.wrap_pecan_controller_exception
    @pecan.expose()
    def post(self, trigger_id):
        """Trigger a new event on Solum."""
        policy.check('create_trigger',
                     pecan.request.security_context)
        commit_sha = ''
        status_url = None
        collab_url = None

        try:
            query = query_dict(pecan.request.query_string)
            workflow = self._get_workflow(query)

            body = json.loads(pecan.request.body)
            if ('sender' in body and 'url' in body['sender'] and
                    'api.github.com' in body['sender']['url']):
                action = body.get('action', None)
                if 'comment' in body:
                    # Process a request for rebuilding
                    commit_sha, collab_url = self._process_request(body)
                elif 'pull_request' in body:
                    # Process a GitHub pull request
                    commit_sha = body['pull_request']['head']['sha']
                else:
                    raise exception.NotImplemented()

                # An example of Github statuses_url
                # https://api.github.com/repos/:user/:repo/statuses/{sha}
                if commit_sha:
                    status_url = body['repository']['statuses_url'].format(
                        sha=commit_sha)
            else:
                # Request NOT from a Github repo
                raise exception.NotImplemented()
        except Exception as exc:
            if isinstance(exc, exception.SolumException):
                raise
            info_msg = "Expected fields not found in request body."
            LOG.info(info_msg)
            raise exception.BadRequest(reason=info_msg)

        try:
            # Trigger workflow only on PR create and on rebuild request
            if action in ['created', 'opened', 'edited', 'reopened',
                          'synchronize', 'closed']:
                handler = app_handler.AppHandler(None)
                handler.trigger_workflow(trigger_id, commit_sha, status_url,
                                         collab_url, workflow=workflow)
        except exception.ResourceNotFound:
            LOG.error("Incorrect trigger url.")
            raise

        pecan.response.status = 202
